from typing import List
from FreeTAKServer.model.protobufModel.fig_pb2 import FederatedEvent
from FreeTAKServer.controllers.serializers.serializer_abstract import SerializerAbstract
from FreeTAKServer.model.FTSModel.Event import Event
from FreeTAKServer.controllers.serializers.xml_serializer import XmlSerializer


class ProtobufSerializer(SerializerAbstract):
    def __init__(self):
        self.attribute_name_mapping = {'coordSource': 'how', 'ploc': '', 'palt': '', 'sendTime': 'time',
                                       'startTime': 'start', 'staleTime': 'stale', 'groupName': '', 'groupRole': '',
                                       'screenName': '', 'phone': '', 'binary': '', 'ptpUids': '',
                                       'battery': '', 'speed': '', 'course': '', 'ptpCallsigns': ''}

    def from_format_to_fts_object(self, object: type, FTSObject: Event) -> Event:
        for descriptor in object.event.DESCRIPTOR.fields:
            attribute_name = descriptor.name

            if attribute_name in self.attribute_name_mapping:
                attribute_name = self.attribute_name_mapping[attribute_name]

            if hasattr(FTSObject.detail, 'marti'):
                from FreeTAKServer.model.FTSModel.Dest import Dest

                for callsign in object.event.ptpCallsigns:
                    newdest = Dest()
                    newdest.setcallsign(callsign)
                    FTSObject.detail.marti.setdest(newdest)

            elif attribute_name != 'other' and attribute_name != '':

                setters = self._get_fts_object_var_setter(FTSObject, attribute_name)
                setter = self._get_method_in_method_list(setters, 'event')

                if attribute_name == 'time' or attribute_name == 'start' or attribute_name == 'stale':
                    import datetime
                    attribute = getattr(object.event, descriptor.name)
                    try:
                        setter(datetime.datetime.strftime(
                            datetime.datetime.strptime(str(datetime.datetime.fromtimestamp(float(attribute) / 1000.0)),
                                                       "%Y-%m-%d %H:%M:%S.%f"), "%Y-%m-%dT%H:%M:%S.%fZ"))
                        continue
                    except:
                        setter(datetime.datetime.strftime(
                            datetime.datetime.strptime(str(datetime.datetime.fromtimestamp(float(attribute) / 1000.0)),
                                                       "%Y-%m-%d %H:%M:%S"), "%Y-%m-%dT%H:%M:%S.%fZ"))
                        continue
                else:
                    setter(getattr(object.event, descriptor.name))

            elif attribute_name == 'other' and object.event.other != '':
                from lxml import etree
                xmldetail = etree.fromstring(object.event.other)
                xmldetail.remove(xmldetail.find('_flow-tags_'))

                if xmldetail.find('remarks'):
                    XmlSerializer().from_format_to_fts_object(etree.tostring(xmldetail).decode(), FTSObject)
                else:
                    xmldetail.append(etree.Element('remarks'))
                    xmldetail.find('remarks').text = 'From federation '
                    XmlSerializer().from_format_to_fts_object(etree.tostring(xmldetail).decode(), FTSObject)

        return FTSObject

    def from_fts_object_to_format(self, FTSObject: Event) -> type:
        try:
            obj = FederatedEvent()
            if hasattr(FTSObject.detail, 'marti'):
                for dest in FTSObject.detail.marti.dest:
                    callsign = dest.getcallsign()
                    if callsign:
                        obj.event.ptpCallsigns.append(callsign)
                    else:
                        pass

            for descriptor in obj.event.DESCRIPTOR.fields:
                attribute_name = descriptor.name

                if attribute_name in self.attribute_name_mapping:
                    attribute_name = self.attribute_name_mapping[attribute_name]

                if attribute_name != 'other' and attribute_name:
                    getters = self._get_fts_object_var_getter(FTSObject, attribute_name)
                    getter = self._get_method_in_method_list(getters, 'event')

                    if attribute_name == 'time' or attribute_name == 'start' or attribute_name == 'stale':
                        import datetime
                        attribute = getter()
                        setattr(obj.event, descriptor.name,
                                int(datetime.datetime.strptime(attribute, "%Y-%m-%dT%H:%M:%S.%fZ").timestamp() * 1000))
                    elif attribute_name in ['lat', 'lon', 'hae', 'ce', 'le', 'speed', 'course']:
                        setattr(obj.event, descriptor.name, float(getter()))
                    elif attribute_name == 'battery':
                        setattr(obj.event, descriptor.name, int(getter()))
                    else:
                        setattr(obj.event, descriptor.name, getter())

                else:
                    from lxml import etree
                    xmldetail = etree.tostring(XmlSerializer().from_fts_object_to_format(FTSObject.detail)).decode()
                    setattr(obj.event, 'other', xmldetail)

            return obj
        except Exception as e:
            raise e
    def _get_method_in_method_list(self, method_list: List[callable], expected_class_name: str) -> callable:
        if len(method_list) == 1:
            return method_list[0]

        elif len(method_list) > 1:
            for method in method_list:
                # required due to pythons privacy conventions
                if method.__self__.__class__.__name__.lower() == expected_class_name.lower():
                    return method
                else:
                    pass
            raise AttributeError(expected_class_name + ' does not have specified attribute')

        else:
            raise AttributeError(expected_class_name + ' does not have specified attribute')



#######################################################
# 
# VideoStream.py
# Python implementation of the Class VideoStream
# Generated by Enterprise Architect
# Created on:      21-Sep-2020 10:23:44 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, func
from FreeTAKServer.model.SQLAlchemy.Root import Base, Root
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import DateTime


class VideoStream(Base, Root):
# default constructor  def __init__(self):  

    __tablename__ = "VideoStreams"
    PrimaryKey = Column(Integer, primary_key = True, autoincrement=True)
    Address = Column(String)
    Alias = Column(String)
    Buffer = Column(Integer, default=-1)
    FullXmlString = Column(String)
    IgnoreEmbeddedKlv = Column(String, default=False)
    Path = Column(String, default=None)
    Port = Column(Integer, default=-1)
    PreferredMacAddress = Column(String, default=None)
    Protocol = Column(String)
    RoverPort = Column(Integer, default=-1)
    RtspReliable = Column(Integer, default=0)
    Timeout = Column(Integer)
    Timestamp = Column(DateTime, default=func.now())


from FreeTAKServer.model.SpecificCoT.SendPrecense import SendPresence
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.model.RestMessages.RestEnumerations import RestEnumerations
from FreeTAKServer.model.FTSModel.Event import Event as event
import json as jsonmodule
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
from FreeTAKServer.controllers.configuration.RestAPIVariables import RestAPIVariables

loggingConstants = LoggingConstants()
logger = CreateLoggerController("SendPresenceController").getLogger()


class SendPresenceController:
    def __init__(self, json):
        tempObject = event.Presence()
        object = SendPresence()
        object.setModelObject(tempObject)
        object.modelObject = self._serializeJsonToModel(object.modelObject, json)
        object.setXmlString(XMLCoTController().serialize_model_to_CoT(object.modelObject))
        self.setCoTObject(object)

    def _serializeJsonToModel(self, object, json):
        try:
            object.sethow(json.gethow())
            object.type = RestAPIVariables.defaultPresenceType
            point = object.point
            point.setlon(json.getlongitude())
            point.setlat(json.getlatitude())
            object.detail.contact.setcallsign(json.getname())
            object.detail._group.setname(json.getteam())
            object.detail._group.setrole(json.getrole())
            if json.gettimeout() != '':
                object.setstale(staletime=int(json.gettimeout()))
            else:
                object.setstale(staletime=RestAPIVariables.defaultPresenceTimeout)
            return object
        except AttributeError as e:
            return Exception('a parameter has been passed which is not recognized with error: '+str(e))

    def setCoTObject(self, CoTObject):
        self.CoTObject = CoTObject

    def getCoTObject(self):
        return self.CoTObject


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# EntityTypes.py
# Python implementation of the Class EntityTypes
# Generated by Enterprise Architect
# Created on:      11-Feb-2020 11:08:09 AM
# Original author: Corvo
# 
#######################################################


class EntityTypes(FTSProtocolObject):
# default constructor       def __init__(self):  

   military = "" 
   

      
   civilian = "" 
   # civilian getter 
   def getcivilian(self): 
      return self.civilian 
 
     # civilian setter 
   def setcivilian(self, civilian=0):  
      self.civilian=civilian 

        # military getter 
   def getmilitary(self): 
      return self.military 

    # military setter 
   def setmilitary(self, military=0):  
      self.military=military 
 
     


import os
from pathlib import PurePath
class LoggingConstants:
    def __init__(self):
        #main logging config
        self.CURRENTPATH = os.path.dirname(os.path.realpath(__file__))
        self.CURRENTPATH = PurePath(self.CURRENTPATH)
        self.PARENTPATH = str(self.CURRENTPATH.parents[0])
        self.LOGFORMAT = '%(levelname)s : %(asctime)s : %(filename)s:%(lineno)d : %(message)s'
        self.LOGNAME = 'FTS'
        self.LOGDIRECTORY = 'logs'
        self.ERRORLOG = str(PurePath(self.PARENTPATH, f"{self.LOGDIRECTORY}/{self.LOGNAME}_error.log"))
        self.DEBUGLOG = str(PurePath(self.PARENTPATH, f"{self.LOGDIRECTORY}/{self.LOGNAME}_debug.log"))
        self.INFOLOG = str(PurePath(self.PARENTPATH, f"{self.LOGDIRECTORY}/{self.LOGNAME}_info.log"))
        self.HTTPLOG = str(PurePath(self.PARENTPATH, f"{self.LOGDIRECTORY}/{self.LOGNAME}_http.log"))
        self.DELIMITER = ' ? '
        self.MAXFILESIZE = 100000
        self.BACKUPCOUNT = 5
        #orchestrator logs
        
        #clientConnected
        self.CLIENTCONNECTED = 'client has connected'
        self.CREATE = 'create'
        self.ADD = 'add'
        self.CLIENTCONNECTEDFINISHED = 'finished establishing connection'
        self.CLIENTCONNECTEDERROR = 'there has been an error in a clients connection'
        
        #emergencyReceived
        self.ON = 'on'
        self.OFF = 'off'
        self.EMERGENCYCREATED = 'emergency has been created'
        self.EMERGENCYREMOVED = 'emergency has now been removed'
        self.EMERGENCY = "emergency"
        self.EMERGENCYRECEIVEDERROR = 'there has been an error in a clients connection'

        #dataReceived
        self.DATARECEIVEDERROR = 'there has been an error in the reception of generic data'

        #clientDisconnected
        self.CLIENTDISCONNECTSTART = 'initiating client disconnection'
        self.REMOVE = "remove"
        self.DESTROY = 'destroy'
        self.CLIENTDISCONNECTEND = 'client disconnection has concluded '
        self.CLIENTDISCONNECTERROR = 'there has been an error in the reception of generic data'

        #monitorRawCoT
        self.MONITORRAWCOTERRORA = 'there has been an error in the reception of data in the monitoring of pipes A'
        self.MONITORRAWCOTERRORB = 'there has been an error in the reception of data in the monitoring of pipes B'
        self.MONITORRAWCOTERRORC = 'there has been an error in the reception of data in the monitoring of pipes C'
        self.MONITORRAWCOTERRORD = 'there has been an error in the reception of data in the monitoring of pipes D'
        self.MONITORRAWCOTERRORINTERNALSCANERROR = 'there has been an error in the scanning of the internal CoT array'

        #start
        self.LOADING = 'loading ...'
        self.SERVERSTARTED = 'server has started'
        self.STARTERROR = 'there has been a critical error in the startup of FTS'
        
        #arguments
        self.ARGUMENTDESCRIPTION = 'FreeTAKServer startup settings'
        self.ARGUMENTCOTPORTDESCRIPTION = 'the port you would like FreeTAKServer to run receive connections on'
        self.ARGUMENTCOTPORT = '-CoTPort'
        self.ARGUMENTCOTPORTDEFAULT = 8087

        self.ARGUMENTCoTIPDESCRIPTION = "the IP you would like FreeTAKServer to run receive connections on ONLY CHANGE IF YOU KNOW WHAT YOU'RE DOING"
        self.ARGUMENTCOTIP = '-CoTIP'
        self.ARGUMENTCoTIPDEFAULT = '0.0.0.0'

        self.ARGUMENTDATAIPDESCRIPTION = 'the ip address you would like FreeTAKServer to run receive datapackages on this is necesarry if its not set correctly data packages will fail'
        self.ARGUMENTDATAIP = '-DataIP'
        self.ARGUMENTDATAIPDEFAULT = '0.0.0.0'

        self.ARGUMENTDATAPORTDESCRIPTION = 'the port you would like FreeTAKServer to run receive connections on'
        self.ARGUMENTDATAPORT = '-DataPort'
        self.ARGUMENTDATAPORTDEFAULT = 8080

        #ClientInformationController
        self.CLIENTINFORMATIONCONTROLLERERROR = 'another exception'

        #ClientReceptionHandler
        #Startup
        self.CLIENTRECEPTIONHANDLERSTART = "client reception handler has finished startup"
        self.CLIENTRECEPTIONHANDLERSTARTUPERROR = 'there has been an error in client reception startup'
       
        #Monitor event pipe
        self.CLIENTRECEPTIONHANDLERMONITOREVENTPIPEERROR = 'there has been an error in a client reception Event Pipe'
        
        #Return data to orchestrator
        self.CLIENTRECEPTIONHANDLERRETURNDATATOORCHESTRATORERROR = 'there has been an error in client reception returning data to the orchestrator'
        
        #create client monitor
        self.CLIENTRECEPTIONHANDLERCREATECLIENTMONITORERROR = 'there has been an error in client reception with the creation of a client monitor'
        self.CLIENTRECEPTIONHANDLERCREATECLIENTMONITORINFO = 'client reception handler thread has finished being created'

        #destroy client monitor
        self.CLIENTRECEPTIONHANDLERDESTROYCLIENTMONITORINFO = 'client reception handler thread has finished being terminated'
        self.CLIENTRECEPTIONHANDLERDESTROYCLIENTMONITORERROR = 'there has been an error in client reception with the destruction of a clients thread '

        #monitorForData
        self.CLIENTRECEPTIONHANDLERMONITORFORDATAERRORA = 'there has been an error in a clients reception thread section A '
        self.CLIENTRECEPTIONHANDLERMONITORFORDATAERRORB = 'there has been an error in a clients reception thread section B '
        self.CLIENTRECEPTIONHANDLERMONITORFORDATAERRORC = 'there has been an error in a clients reception thread section C '
        self.CLIENTRECEPTIONHANDLERMONITORFORDATAERRORD = 'there has been an error in a clients reception thread section D '

        #return Received Data
        self.CLIENTRECEPTIONHANDLERRETURNRECEIVEDDATAERROR = 'there has been an error in a clients reception thread with the returning of received data '

        #data package server

        #ReceiveConnections
        #listen
        self.RECEIVECONNECTIONSLISTENINFO = 'client connected'
        self.RECEIVECONNECTIONSLISTENERROR = 'error in Receive connections listen function '

        #send client data
        #send data in queue

        self.SENDCLIENTDATASENDDATAINQUEUEINFO = 'send data in queue started'

        #Send data controller
        #Send data in queue
        self.SENDDATACONTROLLERSENDDATAINQUEUEERROR = "error send data controller in send data in queue"

        #XMLCoTController
        #determine CoT general
        self.XMLCOTCONTROLLERDETERMINECOTGENERALERRORA = "exception XMLCoTController in determining general CoT type A "
        self.XMLCOTCONTROLLERDETERMINECOTGENERALERRORB = "exception XMLCoTController in determining general CoT type B "


class checklistTask:
    def __init__(self):
        self.lineBreak = "false"
        self.number = 0
        self.uid = ''
        self.status = 'Pending'

    def setuid(self, uid):
        self.uid = uid

    def getuid(self):
        return self.uid

    def getlineBreak(self):
        return self.lineBreak

    def setlineBreak(self, lineBreak):
        self.lineBreak = lineBreak

    def setnumber(self, number):
        self.number = number

    def getnumber(self):
        return self.number

    def setstatus(self, status):
        self.status = status

    def getstatus(self):
        return self.status


#######################################################
# 
# Marti.py
# Python implementation of the Class Marti
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:44 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy.orm import relationship


class Marti(Base):
# default constructor  def __init__(self):
    __tablename__ = "Marti"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    dest = relationship("Dest", backref="owner")



#######################################################
# 
# Federate.py
# Python implementation of the Class Federate
# Generated by Enterprise Architect
# Created on:      12-Sep-2020 10:32:00 PM
# Original author: natha
# 
#######################################################

class Federate:
    """this object contains all pertenent information in relation to one federate
    connected to server service or to which FTS is connected as a client.
    """
# default constructor  def __init__(self):  

    IP = ""  
    Port = "None"   
    Socket = ""
    federationController = None


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.StartTimeVariables import StartTimeVariables as vars

class StartTime(FTSProtocolObject):
    @staticmethod
    def Checklist(INTAG = vars.Checklist().INTAG):
        starttime = StartTime()
        starttime.setINTAG(INTAG)
        return starttime

    def setINTAG(self, INTAG):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG


#######################################################
# 
# ClientInformation.py
# Python implementation of the Class ClientInformation
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:47:19 AM
# Original author: Natha Paquette
# 
#######################################################
import copy

class ClientInformation:
    def __init__(self):  
        self.modelObject = ''
        self.alive = "" 
        self.dataQueue = "" 
        self.ID = "" 
        self.idData = "" 
        self.IP = "" 
        self.receptionHandler = "" 
        self.sendData = "" 
        self.socket = "" 
        self.type = "clientInformation"

    def __deepcopy__(self, memodict={}):
        returned = ClientInformation()
        returned.modelObject = copy.deepcopy(self.modelObject)
        returned.alive = copy.deepcopy(self.alive)
        returned.dataQueue = copy.deepcopy(self.dataQueue)
        returned.ID = copy.deepcopy(self.ID)
        returned.idData = copy.deepcopy(self.idData)
        returned.IP = copy.deepcopy(self.IP)
        returned.receptionHandler = copy.deepcopy(self.receptionHandler)
        returned.sendData = copy.deepcopy(self.sendData)
        returned.type = copy.deepcopy(self.type)
        return returned


from typing import NewType
from FreeTAKServer.model.SpecificCoT.SpecificCoTAbstract import SpecificCoTAbstract
from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject

class Types:
    specific_cot = NewType('specific_cot', SpecificCoTAbstract)
    fts_object = NewType('fts_object', FTSProtocolObject)
    test = NewType('test', int)



#######################################################
# 
# _group.py
# Python implementation of the Class _group
# Generated by Enterprise Architect
# Created on:      28-Sep-2020 10:48:10 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey, String
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy.orm import relationship


class _Group(Base):
# default constructor  def __init__(self):  
    __tablename__ = "_Group"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="_group")
    name = Column(String)
    role = Column(String)


import logging
import os
import random
import string
import traceback
import xml.etree.ElementTree as ET
from logging.handlers import RotatingFileHandler
from pathlib import Path, PurePath
from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants
from FreeTAKServer.controllers.configuration.SQLcommands import SQLcommands
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
from FreeTAKServer.controllers.configuration.DatabaseConfiguration import DatabaseConfiguration
import eventlet
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
from flask_cors import CORS, cross_origin

loggingConstants = LoggingConstants()
logger = CreateLoggerController("DataPackageServer").getLogger()
from flask_sqlalchemy import SQLAlchemy
from flask import Flask, request, send_file
from flask.logging import default_handler

dbController = DatabaseController()

log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)

sql = SQLcommands()
const = DataPackageServerConstants()
log = LoggingConstants()

app = Flask(__name__)  # create the Flask app
app.config['SQLALCHEMY_DATABASE_URI'] = DatabaseConfiguration().DataBaseConnectionString
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.config["SECRET_KEY"] = 'vnkdjnfjknfl1232#'
cors = CORS(app, resources={r"/*": {"origins": "*"}})
app.config['CORS_HEADERS'] = 'Content-Type'

db = SQLAlchemy(app)
# TODO: verify session life cycle in dbController doesnt break this logic
dbController.session = db.session
file_dir = os.path.dirname(os.path.realpath(__file__))
dp_directory = MainConfig.DataPackageFilePath

if not os.path.exists(MainConfig.ExCheckMainPath):
    os.mkdir(MainConfig.ExCheckMainPath)

if not os.path.exists(MainConfig.ExCheckChecklistFilePath):
    os.mkdir(MainConfig.ExCheckChecklistFilePath)

if not os.path.exists(MainConfig.ExCheckFilePath):
    os.mkdir(MainConfig.ExCheckFilePath)
# Set up logging
if not Path(log.LOGDIRECTORY).exists():
    print(f"Creating directory at {log.LOGDIRECTORY}")
    os.makedirs(log.LOGDIRECTORY)
app.logger.removeHandler(default_handler)
formatter = logging.Formatter(log.LOGFORMAT)
file_handler = RotatingFileHandler(
    log.HTTPLOG,
    maxBytes=log.MAXFILESIZE,
    backupCount=log.BACKUPCOUNT
)
file_handler.setFormatter(formatter)
file_handler.setLevel(logging.ERROR)


# app.logger.addHandler(file_handler)
# console_handler = logging.StreamHandler(sys.stdout)
# console_handler.setFormatter(formatter)
# console_handler.setLevel(logging.DEBUG)
# app.logger.addHandler(console_handler)
# app.logger.setLevel(logging.DEBUG)

@app.route('/')
def hello():
    return 'hello world'

@app.route('/Alive')
def alive():
    return 'DataPackage service alive', 200


@app.route("/Marti/vcm", methods=[const.GET])
def get_all_video_links():
    # This is called when the user selects the Download button in the Videos window. It
    # expects an XML listing of all known feeds, so the user can pick and choose which ones
    # to store locally
    try:
        feeds = dbController.query_videostream(column=["FullXmlString"])
        app.logger.info(f"Found {len(feeds)} video feeds in {const.DATABASE}")
        if len(feeds) == 0:
            return ("No video feeds found", 500)
        all_feeds = ""
        for feed in feeds:
            # 'feed' is a tuple with one element, so we only append that
            all_feeds += feed.FullXmlString.decode("utf-8")
        return f"<videoConnections>{all_feeds}</videoConnections>"
    except:
        app.logger.error(traceback.format_exc())
        return "Error", 500


@app.route("/Marti/vcm", methods=[const.POST])
def insert_video_link():
    try:
        xml_root = ET.fromstring(request.data.decode("utf-8"))
        for xml_feed in xml_root:
            protocol = xml_feed.find("protocol").text
            alias = xml_feed.find("alias").text
            uid = xml_feed.find("uid").text
            address = xml_feed.find("address").text
            port = xml_feed.find("port").text
            rover_port = xml_feed.find("roverPort").text
            ignore_klv = xml_feed.find("ignoreEmbeddedKLV").text
            preferred_mac = xml_feed.find("preferredMacAddress").text
            path = xml_feed.find("path").text
            buf = xml_feed.find("buffer").text
            timeout = xml_feed.find("timeout").text
            rtsp_reliable = xml_feed.find("rtspReliable").text
            # Check that no other feeds with the same UID have been received
            streams = dbController.query_videostream(query=f'uid == "{uid}"')
            if len(streams) > 0:
                app.logger.info(f"Already received feed with UID={uid} (alias = {alias})")
                continue  # Ignore this feed if there are duplicates
            app.logger.info(f"Inserting video feed into database: {request.data.decode('utf-8')}")
            dbController.create_videostream(FullXmlString=ET.tostring(xml_feed), Protocol=protocol, Alias=alias,
                                            uid=uid, Address=address, Port=port, RoverPort=rover_port,
                                            IgnoreEmbeddedKlv=ignore_klv, PreferredMacAddress=preferred_mac, Path=path,
                                            Buffer=buf, Timeout=timeout, RtspReliable=rtsp_reliable)

        return "Okay", 200
    except:
        app.logger.error(traceback.format_exc())
        return "Error", 500


@app.route('/Marti/api/version/config', methods=[const.GET])
def versionConfig():
    logger.info('sending client version json')
    return const.VERSIONJSON


@app.route('/Marti/api/clientEndPoints', methods=[const.GET])
def clientEndPoint():
    logger.info('sending client version info')
    return const.versionInfo


@app.route('/Marti/sync/missionupload', methods=[const.POST])
def upload():
    logger.info('dataoackage upload started')
    file_hash = request.args.get('hash')
    app.logger.info(f"Data Package hash = {str(file_hash)}")
    letters = string.ascii_letters
    uid = ''.join(random.choice(letters) for i in range(4))
    uid = 'uid-' + str(uid)
    filename = request.args.get('filename')
    creatorUid = request.args.get('creatorUid')
    file = request.files.getlist('assetfile')[0]
    directory = Path(dp_directory, file_hash)
    if not Path.exists(directory):
        os.mkdir(str(directory))
    file.save(os.path.join(str(directory), filename))
    fileSize = Path(str(directory), filename).stat().st_size
    callsign = str(
        FlaskFunctions().getSubmissionUser(creatorUid,
                                           dbController))  # fetchone() gives a tuple, so only grab the first element
    FlaskFunctions().create_dp(dbController, uid=uid, Name=filename, Hash=file_hash, SubmissionUser=callsign,
                               CreatorUid=creatorUid, Size=fileSize)
    return IP + ':' + str(HTTPPORT) + "/Marti/api/sync/metadata/" + file_hash + "/tool"


@app.route('/Marti/api/sync/metadata/<hash>/tool', methods=[const.PUT])
def putDataPackageTool(hash):
    if request.data == b'private':
        dbController.update_datapackage(query=f'Hash == "{hash}"', column_value={"Privacy": 1})
    return "Okay", 200


@app.route('/Marti/api/sync/metadata/<hash>/tool', methods=[const.GET])
@cross_origin(send_wildcard = True)
def getDataPackageTool(hash):
    from flask import make_response
    file_list = os.listdir(str(dp_directory) + '/' + str(hash))
    path = PurePath(dp_directory, str(hash), file_list[0])
    app.logger.info(f"Sending data package from {str(path)}")
    resp = send_file(str(path))
    return resp


@app.route('/Marti/sync/search', methods=[const.GET])
def retrieveData():
    logger.info('sync search tirggerd')
    keyword = request.args.get('keyword')
    packages = FlaskFunctions().getAllPackages()
    app.logger.info(f"Data packages in the database: {packages}")
    return str(packages)


@app.route('/Marti/sync/content', methods=const.HTTPMETHODS)
def specificPackage():
    from lxml import etree
    from os import listdir
    if request.method == 'GET' and request.args.get('uid') != None:
        data = request.data
        taskuid = request.args.get('uid')
        for file in listdir(MainConfig.ExCheckChecklistFilePath):
            xml = etree.parse(str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(file)))).getroot()

            tasks = xml.find('checklistTasks')
            for task in tasks:
                uid = task.find('uid')
                if str(uid.text) == str(taskuid):
                    return etree.tostring(task)
                else:
                    pass
        for file in listdir(MainConfig.ExCheckFilePath):
            xml = etree.parse(str(PurePath(Path(MainConfig.ExCheckFilePath), Path(file)))).getroot()
            if xml.find("checklistDetails").find('uid').text == str(taskuid):
                return etree.tostring(xml)

    else:
        hash = request.args.get('hash')
        import hashlib
        if os.path.exists(str(PurePath(Path(dp_directory), Path(hash)))):
            logger.info('marti sync content triggerd')
            app.logger.debug(str(PurePath(Path(dp_directory), Path(hash))))
            file_list = os.listdir(str(PurePath(Path(dp_directory), Path(hash))))
            app.logger.debug(PurePath(Path(const.DATAPACKAGEFOLDER), Path(hash), Path(file_list[0])))
            path = PurePath(dp_directory, str(hash), file_list[0])
            app.logger.debug(str(path))
            return send_file(str(path))
        else:
            obj = dbController.query_ExCheck(verbose=True, query=f'hash == "{hash}"')
            data = etree.parse(str(PurePath(Path(MainConfig.ExCheckFilePath), Path(obj[0].data.filename))))
            data.getroot().find('checklistTasks').find("checklistTask").find("uid").text = data.getroot().find('checklistTasks').find("checklistTask").find("checklistUid").text
            output = etree.tostring(data, pretty_print=False)
            return output


@app.route('/Marti/api/version', methods=[const.GET])
def returnVersion():
    logger.info('api version triggered')
    return const.versionInfo


@app.route('/Marti/sync/missionquery', methods=const.HTTPMETHODS)
def checkPresent():
    logger.info('synce missionquery triggered')
    hash = request.args.get('hash')
    if FlaskFunctions().hashIsPresent(hash, dbController):
        app.logger.info(f"Data package with hash {hash} exists")
        return IP + ':' + str(HTTPPORT) + "/Marti/api/sync/metadata/" + hash + "/tool"
    else:
        app.logger.info(f"Data package with hash {hash} does not exist")
        return '404', 404


@app.route('/')
def home():
    return 'data package service is up, good job.'



#exCheckStuff
from flask import Flask, request
from FreeTAKServer.controllers.ExCheckControllers.templateToJsonSerializer import templateSerializer
from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

@app.route('/Marti/api/missions/exchecktemplates/changes', methods=['GET'])
def check_changes():
    try:
        # example return data {"version":"2","type":"MissionChange","data":[],"nodeId":"TAK-Server-a6htdf93"}
        # this endpoint should return any excheck template change since specified time
        # TODO: learn what squached represents
        request.args.get('squached')
        # the time since last connect and the length of time to check for changes
        request.args.get('start')
        return '{"version":"2","type":"MissionChange","data":[{"type":"REMOVE_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T16:44:10.346Z","creatorUid":"CN=testalpha,OU=Dev,O=FTS,L=Yarmouth,ST=NS,C=CA","serverTime":"2020-10-23T16:44:10.366Z","contentResource":{"filename":"d5d2bc15-f6ab-49e7-a408-5f7fe600ca3e.xml","keywords":["test bravo","this is in db","FEATHER"],"mimeType":"application/xml","name":"d5d2bc15-f6ab-49e7-a408-5f7fe600ca3e","submissionTime":"2020-10-20T00:18:57.157Z","submitter":"testbravo","uid":"d5d2bc15-f6ab-49e7-a408-5f7fe600ca3e","hash":"d94a49a958422a451b352b6ebd8b0741780214b1c2da4b0d41e6fca8e3674082","size":2155,"tool":"ExCheck"}},{"type":"REMOVE_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T16:44:10.235Z","creatorUid":"CN=testalpha,OU=Dev,O=FTS,L=Yarmouth,ST=NS,C=CA","serverTime":"2020-10-23T16:44:10.307Z","contentResource":{"hash":"f1197a7ec99442864b26fb58b993869303cd7ad4a7d6139611c0d42cd1ff156b"}},{"type":"REMOVE_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T16:44:10.186Z","creatorUid":"CN=testalpha,OU=Dev,O=FTS,L=Yarmouth,ST=NS,C=CA","serverTime":"2020-10-23T16:44:10.199Z","contentResource":{"filename":"4b730e6a-e2d8-48aa-9b9c-36dc09a0c462.xml","keywords":["ddd","wadasdsa","FEATHER"],"mimeType":"application/xml","name":"4b730e6a-e2d8-48aa-9b9c-36dc09a0c462","submissionTime":"2020-10-23T15:28:59.943Z","submitter":"testbravo","uid":"4b730e6a-e2d8-48aa-9b9c-36dc09a0c462","hash":"19597a47b4be14f9ee527630cf0dc7fb6b6de222502eab41b6c488e4371fb40b","size":1747,"tool":"ExCheck"}},{"type":"REMOVE_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T16:44:10.017Z","creatorUid":"CN=testalpha,OU=Dev,O=FTS,L=Yarmouth,ST=NS,C=CA","serverTime":"2020-10-23T16:44:10.127Z","contentResource":{"hash":"60aac7efcdf2d7001b8ad0d812e1d1528e252e01be130a6f6285b69943f8d633"}},{"type":"REMOVE_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T16:44:09.842Z","creatorUid":"CN=testalpha,OU=Dev,O=FTS,L=Yarmouth,ST=NS,C=CA","serverTime":"2020-10-23T16:44:09.875Z","contentResource":{"filename":"173992a0-d18d-46bb-b178-759011299d60.xml","keywords":["temp","temp 1","FEATHER"],"mimeType":"application/xml","name":"173992a0-d18d-46bb-b178-759011299d60","submissionTime":"2020-10-19T23:21:03.934Z","submitter":"testbravo","uid":"173992a0-d18d-46bb-b178-759011299d60","hash":"41482b4776bb7735654e8ddde2ad02032909b50ab4df5d8de55f99ebe6bc805d","size":1461,"tool":"ExCheck"}},{"type":"REMOVE_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T16:44:07.103Z","creatorUid":"CN=testalpha,OU=Dev,O=FTS,L=Yarmouth,ST=NS,C=CA","serverTime":"2020-10-23T16:44:07.138Z","contentResource":{"hash":"5bc1ac8bdbc28fc13ec002b60215a9c45874384e8fc754286c5d907a80219fc5"}},{"type":"ADD_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T15:28:59.947Z","creatorUid":"S-1-5-21-2720623347-3037847324-4167270909-1002","serverTime":"2020-10-23T15:28:59.963Z","contentResource":{"filename":"4b730e6a-e2d8-48aa-9b9c-36dc09a0c462.xml","keywords":["ddd","wadasdsa","FEATHER"],"mimeType":"application/xml","name":"4b730e6a-e2d8-48aa-9b9c-36dc09a0c462","submissionTime":"2020-10-23T15:28:59.943Z","submitter":"testbravo","uid":"4b730e6a-e2d8-48aa-9b9c-36dc09a0c462","hash":"19597a47b4be14f9ee527630cf0dc7fb6b6de222502eab41b6c488e4371fb40b","size":1747,"tool":"ExCheck"}},{"type":"ADD_CONTENT","missionName":"exchecktemplates","timestamp":"2020-10-23T15:26:45.110Z","creatorUid":"S-1-5-21-2720623347-3037847324-4167270909-1002","serverTime":"2020-10-23T15:26:45.116Z"}],"nodeId":"TAK-Server-560c34e9"}'
    except Exception as e:
        print('exception in check changes' + str(e))

@app.route('/Marti/api/missions/exchecktemplates/subscription', methods=['PUT'])
def request_subscription():
    try:
        # this endpoint allows for the client to request a new subscription
        # possibly the uid of the client db also contains create_time and mission_id
        print(request.args.get('uid'))

        return ('', 200)
    except Exception as e:
        print('exception in request_subscription' + str(e))

@app.route('/Marti/api/missions/exchecktemplates', methods=['GET'])
def exchecktemplates():
    try:
        # when no data available
        # return b'{"version":"2","type":"Mission","data":[{"name":"exchecktemplates","description":"","chatRoom":"","tool":"ExCheck","keywords":[],"creatorUid":"ExCheck","createTime":"2020-10-19T22:37:39.290Z","externalData":[],"uids":[],"contents":[]}],"nodeId":"TAK-Server-560c34e9"}'
        # when data available
        return templateSerializer().convert_object_to_json(DatabaseController().query_ExCheck())
    except Exception as e:
        print(e)
@app.route('/Marti/api/missions/ExCheckTemplates', methods=['GET'])
def ExCheckTemplates():
    try:
        # when no data available
        # return b'{"version":"2","type":"Mission","data":[{"name":"exchecktemplates","description":"","chatRoom":"","tool":"ExCheck","keywords":[],"creatorUid":"ExCheck","createTime":"2020-10-19T22:37:39.290Z","externalData":[],"uids":[],"contents":[]}],"nodeId":"TAK-Server-560c34e9"}'
        # when data available
        return templateSerializer().convert_object_to_json(DatabaseController().query_ExCheck())
    except Exception as e:
        print(e)

@app.route('/Marti/api/missions/<templateuid>/subscription', methods=['DELETE', 'PUT'])
def missionupdate(templateuid):
    from flask import request
    uid = request.args.get('uid')
    return '', 200

@app.route('/Marti/api/excheck/template', methods=['POST'])
def template():
    try:
        import uuid
        xmlstring = f'<?xml version="1.0"?><event version="2.0" uid="{uuid.uuid4()}" type="t-x-m-c" time="2020-11-28T17:45:51.000Z" start="2020-11-28T17:45:51.000Z" stale="2020-11-28T17:46:11.000Z" how="h-g-i-g-o"><point lat="0.00000000" lon="0.00000000" hae="0.00000000" ce="9999999" le="9999999" /><detail><mission type="CHANGE" tool="ExCheck" name="exchecktemplates" authorUid="S-1-5-21-2720623347-3037847324-4167270909-1002"><MissionChanges><MissionChange><contentResource><filename>61b01475-ad44-4300-addc-a9474ebf67b0.xml</filename><hash>018cd5786bd6c2e603beef30d6a59987b72944a60de9e11562297c35ebdb7fd6</hash><keywords>test init</keywords><keywords>dessc init</keywords><keywords>FEATHER</keywords><mimeType>application/xml</mimeType><name>61b01475-ad44-4300-addc-a9474ebf67b0</name><size>1522</size><submissionTime>2020-11-28T17:45:47.980Z</submissionTime><submitter>wintak</submitter><tool>ExCheck</tool><uid>61b01475-ad44-4300-addc-a9474ebf67b0</uid></contentResource><creatorUid>S-1-5-21-2720623347-3037847324-4167270909-1002</creatorUid><missionName>exchecktemplates</missionName><timestamp>2020-11-28T17:45:47.983Z</timestamp><type>ADD_CONTENT</type></MissionChange></MissionChanges></mission></detail></event>'
        # this is where the client will post the xmi of a template
        from flask import request
        from datetime import datetime
        from lxml import etree
        import hashlib
        # possibly the uid of the client submitting the template
        Y = request
        uid = request.args.get('clientUid')
        XMI = request.data.decode()
        serializer = templateSerializer(XMI)
        object = serializer.convert_template_to_object()
        object.timestamp = datetime.strptime(object.timestamp, "%Y-%m-%dT%H:%M:%S.%fZ")
        serializer.create_DB_object(object)
        xml = etree.fromstring(XMI)
        tasks = xml.find('checklistTasks')
        path = str(PurePath(Path(MainConfig.ExCheckFilePath), Path(f'{object.data.uid}.xml')))
        with open(path, 'w+') as file:
            file.write(XMI)
            file.close()

        uid = object.data.uid
        temp = etree.fromstring(XMI)
        cot = etree.fromstring(xmlstring)
        resources = cot.find('detail').find('mission').find('MissionChanges').find('MissionChange').find('contentResource')
        resources.find('filename').text = temp.find('checklistDetails').find('uid').text + '.xml'
        resources.findall('keywords')[0].text = temp.find('checklistDetails').find('name').text
        resources.findall('keywords')[1].text = temp.find('checklistDetails').find('description').text
        resources.findall('keywords')[2].text = temp.find('checklistDetails').find('creatorCallsign').text
        resources.find('uid').text = temp.find('checklistDetails').find('uid').text
        resources.find('name').text = temp.find('checklistDetails').find('uid').text
        resources.find('size').text = str(len(XMI))
        resources.find('hash').text = str(hashlib.sha256(str(XMI).encode()).hexdigest())
        z = etree.tostring(cot)
        from FreeTAKServer.model.testobj import testobj
        object = testobj()
        object.xmlString = z
        PIPE.send(object)
        return str(uid), 200
    except Exception as e:
        print(str(e))

@app.route('/Marti/api/excheck/<subscription>/start', methods=['POST'])
def startList(subscription):
    import uuid
    from lxml import etree
    import datetime
    uid = str(uuid.uuid4())
    r = request
    # client uid
    request.args.get('clientUid')
    # name of template
    request.args.get('name')
    # description of template
    request.args.get('description')
    # startTime of template
    startTime = request.args.get('startTime')
    # callsign of submission user
    request.args.get('callsign')

    with open(str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(f'{uid}.xml'))), 'w+') as file:
        file.write(str(open(str(PurePath(Path(MainConfig.ExCheckFilePath), Path(f'{subscription}.xml'))), 'r').read()))
        file.close()

    xml = etree.parse(
        MainConfig.ExCheckChecklistFilePath + '/' + uid + '.xml').getroot()

    starttime = etree.Element('startTime')
    starttime.text = startTime
    details = xml.find('checklistDetails')
    if details.find('startTime') == None:

        details.append(starttime)
    else:
        details.find('startTime').text = startTime
    uids = details.find('uid')
    uids.text = uid
    details.find('description').text = request.args.get('description')
    details.find('name').text = request.args.get('name')

    tasks = xml.find('checklistTasks')
    for task in tasks:
        taskuid = task.find('uid')
        taskuid.text = str(uuid.uuid4())


    with open(
            str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(uid + '.xml'))),
            'w+') as file:
        y = etree.tostring(xml)
        file.write(etree.tostring(xml).decode())
        file.close()

    excheckobj = dbController.query_ExCheck(f'ExCheckData.uid == "{subscription}"', verbose=True,)[0]
    dbController.create_Excheckchecklist(startTime=datetime.datetime.strptime(startTime, '%Y-%m-%dT%H:%M:%S.%fZ'), creatorUid = request.args.get('clientUid'), description = request.args.get('description'), callsign = request.args.get('callsign'), name = request.args.get('name'), uid = uid, filename = f'{uid}.xml', template = excheckobj)

    return str(open(str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(uid + '.xml'))), 'r').read()), 200

@app.route('/Marti/api/excheck/checklist/<checklistid>')
def accesschecklist(checklistid):
    return str(open(str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(checklistid + '.xml'))),
              'r').read())

@app.route('/Marti/api/excheck/checklist/<checklistid>/task/<taskid>', methods=['PUT'])
def updatetemplate(checklistid, taskid):
    from flask import request
    from lxml import etree
    from FreeTAKServer.controllers.SpecificCoTControllers.SendExcheckUpdateController import SendExcheckUpdateController
    from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
    from FreeTAKServer.model.FTSModel.Event import Event
    from FreeTAKServer.model.RawCoT import RawCoT
    import uuid
    import hashlib


    data = request.data

    xml = etree.parse(
        str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(checklistid + '.xml')))).getroot()
    updatedTask = etree.fromstring(data)
    tasks = xml.find('checklistTasks')
    for task in tasks:
        uid = task.find('uid')
        if uid.text == taskid:
            tasks.replace(task, updatedTask)
        else:
            pass
    with open(
            str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(checklistid + '.xml'))), 'w+') as file:
        file.write(etree.tostring(xml).decode())
        file.close()

    # Create Object to send to client
    object = Event.ExcheckUpdate()
    object.setuid(str(uuid.uuid4()))
    object.setversion('2.0')
    object.detail.mission.settype("CHANGE")
    object.detail.mission.settool("ExCheck")
    object.detail.mission.setname(checklistid)
    object.detail.mission.setauthorUid(request.args.get("clientUid"))
    object.detail.mission.MissionChanges.MissionChange.creatorUid.setINTAG(request.args.get("clientUid"))
    object.detail.mission.MissionChanges.MissionChange.missionName.setINTAG(checklistid)
    object.detail.mission.MissionChanges.MissionChange.type.setINTAG("ADD_CONTENT")
    object.detail.mission.MissionChanges.MissionChange.contentResource.filename.setINTAG(taskid+'.xml')
    object.detail.mission.MissionChanges.MissionChange.contentResource.hash.setINTAG(str(hashlib.sha256(str(open(MainConfig.ExCheckChecklistFilePath + '/' + checklistid + '.xml', 'r')).encode()).hexdigest()))
    object.detail.mission.MissionChanges.MissionChange.contentResource.keywords.setINTAG('Task')
    object.detail.mission.MissionChanges.MissionChange.contentResource.name.setINTAG(taskid)
    object.detail.mission.MissionChanges.MissionChange.contentResource.size.setINTAG(str(len(data)))
    #TODO: change this value
    object.detail.mission.MissionChanges.MissionChange.contentResource.submitter.setINTAG('atak')
    object.detail.mission.MissionChanges.MissionChange.contentResource.uid.setINTAG(taskid)



    '''object = etree.fromstring(templateex)
    object.uid = uuid.uuid4()
    object.find('detail').find('mission').type= "CHANGE"
    object.find('detail').find('mission').name = taskid
    object.find('detail').find('mission').Uid = request.args.get("clientUid")
    object.find('detail').find('mission').find('MissionChanges').find('MissionChange').find('creatorUid').text = request.args.get("clientUid")
    object.find('detail').find('mission').find('MissionChanges').find('MissionChange').find('missionName').text = taskid
    object.find('detail').find('mission').find('MissionChanges').find('MissionChange').find('filename').text = checklistid+'.xml'
    object.detail.mission.MissionChanges.MissionChange.contentResource.hash.setINTAG(str(hashlib.sha256(str(data).encode()).hexdigest()))
    object.detail.mission.MissionChanges.MissionChange.contentResource.keywords.setINTAG('Task')
    object.detail.mission.MissionChanges.MissionChange.contentResource.name.setINTAG(checklistid)
    object.detail.mission.MissionChanges.MissionChange.contentResource.size.setINTAG(str(len(data)))
    #TODO: change this value
    object.detail.mission.MissionChanges.MissionChange.contentResource.submitter.setINTAG('test')
    object.detail.mission.MissionChanges.MissionChange.contentResource.uid.setINTAG(checklistid)'''
    rawcot = RawCoT()
    xml = XMLCoTController().serialize_model_to_CoT(object)
    rawcot.xmlString = xml

    PIPE.send(rawcot)
    #PIPE.send()

    return '', 200

@app.route('/Marti/sync/content')
def sync():
    # this endpoint was triggered on attempting to create new template from existing template
    # likely the hash of the excheck
    y = request
    request.args.get('hash')
    uid = request.args.get('uid')
    return '', 200

@app.route('/Marti/api/excheck/checklist/active', methods=["GET"])
def activechecklists():
    from os import listdir
    from FreeTAKServer.model.FTSModel.Checklists import Checklists
    from FreeTAKServer.model.FTSModel.Checklist import Checklist
    from lxml import etree
    checklists = Checklists.Checklist()
    rootxml = etree.Element('checklists')

    for file in listdir(MainConfig.ExCheckChecklistFilePath):
        checklist = etree.Element('checklist')
        xmldetails = etree.parse(str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(file)))).getroot().find('checklistDetails')
        checklist.append(xmldetails)
        checklist.append(etree.Element('checklistColumns'))
        checklist.append(etree.Element('checklistTasks'))
        rootxml.append(checklist)

    xml = etree.tostring(rootxml, pretty_print=False)
    return xml
class FlaskFunctions:

    def __init__(self):
        self.callsigns = []

    def create_dp(self, dbController, **args):
        return dbController.create_datapackage(**args)

    def hashIsPresent(self, hash, dbControl):
        data = dbControl.query_datapackage(query=f'Hash == "{hash}"')
        return len(data) > 0

    def getSubmissionUser(self, UID, dbControl):
        callsign = dbControl.query_user(query=f'uid == "{UID}"', column=['callsign'])
        return callsign

    def getAllPackages(self):
        data = DatabaseController().query_datapackage("Privacy == 0")
        package_dict = {
            "resultCount": len(data),
            "results": []
        }
        for i in data:
            package_dict["results"].append({
                "UID": i.uid,
                "Name": i.Name,
                "Hash": i.Hash,
                "PrimaryKey": i.PrimaryKey,
                "SubmissionDateTime": str(i.SubmissionDateTime.strftime("%Y-%m-%dT%H:%M:%S.%fZ")),
                "SubmissionUser": i.SubmissionUser,
                "CreatorUid": i.CreatorUid,
                "Keywords": i.Keywords,
                "MIMEType": i.MIMEType,
                "Size": i.Size
            })
        return package_dict

    def startup(self, ip, port, pipe):
        try:
            from eventlet import wsgi

            global IP, HTTPPORT
            IP = ip
            HTTPPORT = port
            # Make sure the data package directory exists
            if not Path(dp_directory).exists():
                app.logger.info(f"Creating directory at {str(dp_directory)}")
                os.makedirs(str(dp_directory))
            # Create the relevant database tables
            print(const.IP)
            print(HTTPPORT)
            # app.run(host='0.0.0.0', port=8080)
            wsgi.server(eventlet.listen((DataPackageServerConstants().IP, int(HTTPPORT))), app)

        except Exception as e:
            logger.error('there has been an exception in Data Package service startup ' + str(e))
            return -1

    def stop(self):
        func = request.environ.get('werkzeug.server.shutdown')
        if func is None:
            raise RuntimeError('Not running with the Werkzeug Server')
        func()

    def setIP(self, IP_to_be_set):
        global IP
        IP = IP_to_be_set

    def getIP(self):
        global IP
        return IP

    def setHTTPPORT(self, HTTPPORTToBeSet):
        global HTTPPORT
        HTTPPORT = HTTPPORTToBeSet

    def getHTTPPort(self):
        global HTTPPORT
        return HTTPPORT

    def setPIPE(self, PIPEtoBeSet):
        global PIPE
        PIPE = PIPEtoBeSet

    def getPIPE(self):
        global PIPE
        return PIPE
if __name__ == "__main__":
    pass



#######################################################
# 
# Detail.py
# Python implementation of the Class Detail
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:25 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy.orm import relationship
import FreeTAKServer.model.SQLAlchemy.CoTTables.Archive
import FreeTAKServer.model.SQLAlchemy.CoTTables._Group
import FreeTAKServer.model.SQLAlchemy.CoTTables.Chat
import FreeTAKServer.model.SQLAlchemy.CoTTables.Color
import FreeTAKServer.model.SQLAlchemy.CoTTables.Contact
import FreeTAKServer.model.SQLAlchemy.CoTTables.Dest
import FreeTAKServer.model.SQLAlchemy.CoTTables.Emergency
import FreeTAKServer.model.SQLAlchemy.CoTTables.Link
import FreeTAKServer.model.SQLAlchemy.CoTTables.Marti
import FreeTAKServer.model.SQLAlchemy.CoTTables.Precisionlocation
import FreeTAKServer.model.SQLAlchemy.CoTTables.Remarks
import FreeTAKServer.model.SQLAlchemy.CoTTables.Serverdestination
import FreeTAKServer.model.SQLAlchemy.CoTTables.Status
import FreeTAKServer.model.SQLAlchemy.CoTTables.Summary
import FreeTAKServer.model.SQLAlchemy.CoTTables.Takv
import FreeTAKServer.model.SQLAlchemy.CoTTables.Track
import FreeTAKServer.model.SQLAlchemy.CoTTables.Uid
import FreeTAKServer.model.SQLAlchemy.CoTTables.Usericon



class Detail(Base):
# default constructor  def __init__(self):
    __tablename__ = 'Detail'
    PrimaryKey = Column(ForeignKey("Event.uid"), primary_key=True)
    # Event = relationship("Event", back_populates="detail")
    xmlString = Column(String)
    archive = relationship("Archive", uselist=False, cascade="all, delete")
    _group = relationship("_Group", uselist=False, back_populates="Detail", cascade="all, delete")
    chat = relationship("Chat", uselist=False, cascade="all, delete")
    color = relationship("Color", uselist=False, cascade="all, delete")
    contact = relationship("Contact", back_populates="Detail", uselist=False, cascade="all, delete")
    emergency = relationship("Emergency", uselist=False, cascade="all, delete")
    link = relationship("Link", back_populates="Detail", uselist=False, cascade="all, delete")
    marti = relationship("Marti", uselist=False, cascade="all, delete")
    precisionlocation = relationship("Precisionlocation", uselist=False, cascade="all, delete")
    remarks = relationship("Remarks", uselist=False, cascade="all, delete")
    serverdestination = relationship("Serverdestination", uselist=False, cascade="all, delete")
    status = relationship("Status", uselist=False, cascade="all, delete")
    summary = relationship("Summary", back_populates="Detail", uselist=False, cascade="all, delete")
    takv = relationship("Takv", uselist=False, cascade="all, delete")
    track = relationship("Track", uselist=False, cascade="all, delete")
    uid = relationship("Uid", uselist=False, cascade="all, delete")
    usericon = relationship("Usericon", uselist=False, cascade="all, delete")




from .DataPackageServer import FlaskFunctions, Path, dp_directory, os, app, eventlet, const
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants

loggingConstants = LoggingConstants()
logger = CreateLoggerController("TCPDataPackageServer").getLogger()

class TCPDataPackageService(FlaskFunctions):
    def startup(self, ip, port, pipe):
        try:
            from eventlet import wsgi
            global IP, HTTPPORT, PIPE
            IP = ip
            HTTPPORT = port
            PIPE = pipe
            # Make sure the data package directory exists
            if not Path(dp_directory).exists():
                app.logger.info(f"Creating directory at {str(dp_directory)}")
                os.makedirs(str(dp_directory))
            # Create the relevant database tables
            print(const.IP)
            print(HTTPPORT)
            super().setHTTPPORT(HTTPPORT)
            super().setIP(IP)
            super().setPIPE(PIPE)
            wsgi.server(eventlet.listen((DataPackageServerConstants().IP, HTTPPORT)), app)


        except Exception as e:
            logger.error('there has been an exception in Data Package service startup ' + str(e))
            return -1


#######################################################
# 
# RawCoT.py
# Python implementation of the Class RawCoT
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:47:30 AM
# Original author: Natha Paquette
# 
#######################################################


class RawCoT:
    def __init__(self):  
        self.type = "RawCoT"
        self.CoTType = "" 
        self.clientInformation = ""
        self.xmlString = b""
        self.disconnect = 0
        self.status = ''
        self.protocol = 'xml'



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
class Archive(FTSProtocolObject):
    def __init__(self):
        pass

    @staticmethod
    def drop_point():
        pass


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModel.Checklist import Checklist

class Checklists(FTSProtocolObject):
    def __init__(self):
        self.checklist = []
        self.__index = 0
    @staticmethod
    def Checklist():
        checklists = Checklists()
        checklists.checklist = []
        return checklists

    def setchecklist(self, checklistitem):
        self.checklist.append(checklistitem)

    def getchecklist(self):
        item = self.__index
        self.__index += 1
        return self.checklist[item]



import socket
import time
#TODO: remove strings
class HealthCheckController:
    def __init__(self, port, ip):
        self.connectionString = b'<?xml version="1.0" encoding="UTF-8"?><event><detail><connection conn="True"/></detail></event>'
        self.dataString = b'<?xml version="1.0" encoding="UTF-8"?><event><detail><healthCheck healthy="True"/></detail></event>'
        self.ip = ip
        self.port = port
        self.health = []
        connection = self.establishConnection()
        if connection == 'connection successful':
            dataRecieved = self.receiveData()
            if dataRecieved == True:
                self.health = [1, 1]
            else:
                self.health = [1, 0]
        else:
            self.health = [0,0]
        self.s.send(b'')
    def establishConnection(self):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.s.connect((self.ip, self.port))
        except:
            return 'connection failed'
        self.s.send(self.connectionString)
        self.s.recv(2048)
        return 'connection successful'

    def receiveData(self):
        try:
            timeout = 30
            time.sleep(2)
            self.s.send(self.dataString)
            timeout_start = time.time()
            foundData = False
            self.s.settimeout(10)
            while time.time() < timeout_start+timeout:
                data = self.s.recv(2048)
                if data == self.dataString:
                    foundData = True
                    break
                else:
                    pass
            return foundData
        except:
            return False
        self.s.send(b'')
        self.s.close()
    def returnHealth(self):
        return self.health


from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
from lxml import etree
from FreeTAKServer.model.ExCheck.templateInstanceContents import templateInstanceContents
import hashlib
import os
from FreeTAKServer.model.ExCheck.template import template
from FreeTAKServer.model.ExCheck.templateInstance import templateInstance

class templateSerializer:
    def __init__(self, templateFile = None):
        self.templatedata = templateFile

    def convert_template_to_object(self):
        templateObject = templateInstanceContents()
        templateDetails = etree.fromstring(self.templatedata)
        templateDetails = templateDetails.find('checklistDetails')
        templateObject.settimestamp()
        templateObject.data.setkeywords(name=templateDetails.find('name').text, description=templateDetails.find('description').text, callsign=templateDetails.find('creatorCallsign').text)
        templateObject.setcreatoruid(templateDetails.find('creatorUid').text)
        templateObject.data.setname(templateDetails.find('uid').text)
        templateObject.data.setuid(templateDetails.find('uid').text)
        templateObject.data.sethash(str(hashlib.sha256(str(self.templatedata).encode()).hexdigest()))
        templateObject.data.setsize(len(self.templatedata))
        templateObject.data.setfilename()
        # TODO: include in DB the submitting user
        templateObject.data.setsubmitter('test')
        templateObject.data.settool('ExCheck')
        templateObject.data.setsubmissionTime()
        return templateObject

    def convert_DB_to_object(self, DBObject):
        templateObject = templateInstanceContents()
        templateObject.setcreatoruid(DBObject.creatorUid)
        templateObject.data.setname(DBObject.data.uid)
        templateObject.data.setuid(DBObject.data.uid)
        templateObject.data.sethash(DBObject.data.hash)
        templateObject.data.setsize(DBObject.data.size)
        templateObject.data.setkeywords(name=DBObject.data.keywords.name, description=DBObject.data.keywords.description, callsign=DBObject.data.keywords.callsign)
        templateObject.data.setfilename()
        # TODO: include in DB the submitting user
        templateObject.data.setsubmitter('test')
        templateObject.data.settool('ExCheck')
        templateObject.data.setsubmissionTime()
        return templateObject

    def create_DB_object(self, templateObject):
        self.DataBase = DatabaseController()
        output = self.DataBase.create_ExCheck(templateObject)
        self.DataBase.shutdown_Connection()
        return output
    def convert_object_to_json(self, DBObject):
        templateJsonMessage = template()
        templateJsonMessage.data.append(templateInstance())
        for content in DBObject:
            contentobj = self.convert_DB_to_object(content)
            contentobj.data = vars(contentobj.data)
            templateJsonMessage.data[0].contents.append(vars(contentobj))
        y = vars(templateJsonMessage)
        y['data'][0] = vars(y['data'][0])
        return y

if __name__ == "__main__":
    serializer = templateSerializer(r'C:\Users\natha\PycharmProjects\InDev\FreeTAKServerParent\FreeTAKServer\Tests\ExCheckTesting\excheckxmi\13c3e6b4-c25a-40d3-9938-0d32a6c09e45.xml')
    obj1 = serializer.convert_template_to_object()
    #obj2 = serializer.create_DB_object(obj1)
    #serializer.convert_object_to_json(DatabaseController().query_ExCheck())


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.SubmitterVariables import SubmitterVariables as vars

class Submitter(FTSProtocolObject):
    def __init__(self):
        self.INTAG = None

    @staticmethod
    def ExcheckUpdate(INTAG=vars.ExcheckUpdate().INTAG, ):
        submitter = Submitter()

        submitter.setINTAG(INTAG)

        return submitter

    def setINTAG(self, INTAG):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG


from .SpecificCoTAbstract import SpecificCoTAbstract

class SendEmergency(SpecificCoTAbstract):
    def __init__(self):
        self.define_variables()
        self.setType('Emergency')
        self.setPlaceInternalArray(True)
        self.setStatus('')

    def setPlaceInternalArray(self, placeInternalArray):
        self.placeInternalArray = placeInternalArray

    def getPlaceInternalArray(self):
        return self.placeInternalArray

    def setStatus(self, status):
        self.status = status

    def getStatus(self):
        return self.status



#######################################################
# 
# DataPackage.py
# Python implementation of the Class DataPackage
# Generated by Enterprise Architect
# Created on:      21-Sep-2020 10:28:02 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column
from FreeTAKServer.model.SQLAlchemy.Root import Base, Root
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import VARCHAR
from sqlalchemy import CHAR
from sqlalchemy import DateTime
from datetime import datetime as dt

class DataPackage(Base, Root):
    """
    sqlalchemy object of table of datapackages and their information
    """

    # Name of the table
    __tablename__ = "DataPackages"

    #primary key
    PrimaryKey = Column(Integer, primary_key = True, autoincrement=True)

    # UID of Package Creator
    CreatorUid = Column(String)

    # 32-bit hash of datapackage
    Hash = Column(VARCHAR)

    # data package keywords
    Keywords = Column(CHAR, default='foobar')

    # File type
    MIMEType = Column(String, default="[application/x-zip-compressed]")

    # Name of datapackage
    Name = Column(String)

    # whether or not the data package is private 0 indicates false meaning it will be publicly broadcasted.
    Privacy = Column(Integer, default=0)

    # the size in bytes of the datapackage
    Size = Column(Integer)

    # the date time the datapackage was submitted
    SubmissionDateTime = Column(DateTime, default=dt.utcnow())

    # the call sign of the user who submitted the data package
    SubmissionUser = Column(String)



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.TimestampVariables import TimestampVariables as vars
import datetime as dt

class Timestamp(FTSProtocolObject):
    def __init__(self):
        self.INTAG = None

    @staticmethod
    def ExcheckUpdate(INTAG=vars.ExcheckUpdate().INTAG, ):
        timestamp = Timestamp()

        timestamp.setINTAG(INTAG)

        return timestamp

    def setINTAG(self, INTAG=None):
        DATETIME_FMT = "%Y-%m-%dT%H:%M:%S.%fZ"
        if INTAG == None:
            timer = dt.datetime
            now = timer.utcnow()
            zulu = now.strftime(DATETIME_FMT)
            self.INTAG = zulu
        else:
            self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG


from FreeTAKServer.controllers.DatabaseControllers.table_controllers import TableController
from FreeTAKServer.model.SQLAlchemy.DataPackage import DataPackage


class DataPackageTableController(TableController):
		
	def __init__(self):
		self.table = DataPackage



class statusVariables:
    def __init__(self):
        self.BATTERY = None
        self.READINESS = None

    @classmethod
    def drop_point(cls):
        cls.BATTERY = None
        cls.READINESS = "true"
        return cls

    @classmethod
    def connection(cls):
        cls.BATTERY = '100'
        cls.READINESS = "true"
        return cls

    @classmethod
    def UserUpdate(cls):
        cls.battery = None
        return cls



class checklistDetails:
    def __init__(self):
        self.name = ''
        self.uid = ''
        self.description = ''
        self.startTime = ''
        self.creatorUid = ''
        self.creatorCallsign = ''

    def setname(self, name = None):
        self.name = name

    def getname(self):
        return self.name

    def setcreatorUid(self, creatoruid):
        self.creatorUid = creatoruid

    def getcreatorUid(self):
        return self.creatorUid

    def setcreatorCallsign(self, creatorCallsign):
        self.creatorCallsign = creatorCallsign

    def getcreatorCallsign(self):
        return self.creatorCallsign

    def setuid(self, uid):
        self.uid = uid

    def getuid(self):
        return self.uid

    def setstartTime(self, startTime):
        self.startTime = startTime

    def getstartTime(self):
        return self.startTime


class FilterGroup:
    def __init__(self):
        # an array of pipes to receive data from
        self.sources = []

        # an array of pipes to send data too
        self.receivers = []

        # an array of types of CoT which can be received by this group
        self.allowedType = ['*']

        # an array of types of CoT which can't be received by this group
        self.rejectedType = []

    def add_source(self, source):
        self.sources.append(source)
    
    def remove_source(self, source):
        self.sources.remove(source)
        
    def get_sources(self):
        return self.receivers

    def add_receiver(self, source):
        self.receivers.append(source)
    
    def remove_receiver(self, source):
        self.receivers.remove(source)
        
    def get_receivers(self):
        return self.receivers
    
    def add_allowed_type(self, type):
        self.allowedType.append(type)

    def remove_allowed_type(self, type):
        self.allowedType.remove(type)

    def get_allowed_types(self):
        return self.allowedType

    def check_if_type_is_allowed(self, type):
        if type in self.allowedType and type not in self.rejectedType or self.allowedType == ['*'] and type not in self.rejectedType:
            return True

        else:
            return False


from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class TCPDataPackageServiceVariables:
    def __init__(self):
        self.TCPDataPackageServiceIP = MainConfig.DataPackageServiceDefaultIP
        self.TCPDataPackageServicePort = 8080
        self.TCPDataPackageServiceStatus = ""


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.DescriptionVariables import DescriptionVariables as vars

class Description(FTSProtocolObject):
    @staticmethod
    def Checklist(INTAG = vars.Checklist().INTAG):
        description = Description()
        description.setINTAG(INTAG)
        return description

    def setINTAG(self, INTAG):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# DimensionTypes.py
# Python implementation of the Class DimensionTypes
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):    11Feb2020 11:08:09 AM
# Original author: Corvo
# 
#######################################################


class DimensionTypes(FTSProtocolObject):
# default constructor    def __init__(self):  

    space = "p" 
    # space getter 
    def getspace(self): 
        return self.space 
 
    # space setter 
    def setspace(self,space=0):  
        self.space=space 
 
    air = "A" 
    # air getter 
    def getair(self): 
        return self.air 
 
    # air setter 
    def setair(self,air=0):  
        self.air=air 
 
    landunit = "G" 
    # landunit getter 
    def getlandunit(self): 
        return self.landunit 
 
    # landunit setter 
    def setlandunit(self,landunit=0):  
        self.landunit=landunit 
 
    landequipment = "G" 
    # landequipment getter 
    def getlandequipment(self): 
        return self.landequipment 
 
    # landequipment setter 
    def setlandequipment(self,landequipment=0):  
        self.landequipment=landequipment 
 
    landinstallation = "G" 
    # landinstallation getter 
    def getlandinstallation(self): 
        return self.landinstallation 
 
    # landinstallation setter 
    def setlandinstallation(self,landinstallation=0):  
        self.landinstallation=landinstallation 
 
    seasurface = "S" 
    # seasurface getter 
    def getseasurface(self): 
        return self.seasurface 
 
    # seasurface setter 
    def setseasurface(self,seasurface=0):  
        self.seasurface=seasurface 
 
    seasubsurface = "U" 
    # seasubsurface getter 
    def getseasubsurface(self): 
        return self.seasubsurface 
 
    # seasubsurface setter 
    def setseasubsurface(self,seasubsurface=0):  
        self.seasubsurface=seasubsurface 
 
    subsurface = "U" 
    # subsurface getter 
    def getsubsurface(self): 
        return self.subsurface 
 
    # subsurface setter 
    def setsubsurface(self,subsurface=0):  
        self.subsurface=subsurface 
 
    other = "X" 
    # other getter 
    def getother(self): 
        return self.other 
 
    # other setter 
    def setother(self,other=0):  
        self.other=other 
    


#######################################################
# 
# Teams.py
# Python implementation of the Enumeration Teams
# Generated by Enterprise Architect
# Created on:      04-Nov-2020 4:43:17 PM
# Original author: natha
# 
#######################################################


class Teams:
# default constructor  def __init__(self):  
    # default constructor  def __init__(self):  


    Yellow = "" 
    Red = "" 
    Green = "" 
    Blue = "" 



from FreeTAKServer.model.SpecificCoT.SendGeoChat import SendGeoChat
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.model.RestMessages.RestEnumerations import RestEnumerations
from FreeTAKServer.model.FTSModel.Event import Event as event
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController

loggingConstants = LoggingConstants()
logger = CreateLoggerController("SendGeoChatController").getLogger()

class SendChatController:
    def __init__(self, json):
        tempObject = event.GeoChat()
        object = SendGeoChat()
        object.setModelObject(tempObject)
        object.modelObject = self._serializeJsonToModel(object.modelObject, json)
        object.setXmlString(XMLCoTController().serialize_model_to_CoT(object.modelObject))
        self.setCoTObject(object)

    def _serializeJsonToModel(self, object, json):
        object.detail.remarks.setINTAG(json.getmessage())
        object.detail.remarks.setsource(json.getsender())
        object.detail.link.setuid(json.getsender())
        return object

    def setCoTObject(self, CoTObject):
        self.CoTObject = CoTObject

    def getCoTObject(self):
        return self.CoTObject


from FreeTAKServer.model.ServiceObjects.FTS import FTS
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
import typing

def modifyDefaultIP(func):
    def changeDefaultIP(instance, port, ip):
        import socket
        if ip == "0.0.0.0":
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('10.255.255.255', 1))
            IP = s.getsockname()[0]
            ip = IP
            func(instance, port, ip)
        else:
            func(instance, port, ip)
    return changeDefaultIP

class ServerStatusController:

    def __init__(self, ServerStatusObject: FTS):
        mapping = {"start": "on", "stop": "off", "": ""}
        self.TCPCoTStatus = self.TCPCoTStatusCheck(ServerStatusObject.CoTService.CoTServicePort, ServerStatusObject.CoTService.CoTServiceIP)
        self.TCPCoTStatusExpected = mapping[ServerStatusObject.CoTService.CoTServiceStatus]

        self.TCPDataPackageStatus = self.TCPDataPackageStatusCheck(ServerStatusObject.TCPDataPackageService.TCPDataPackageServicePort, ServerStatusObject.TCPDataPackageService.TCPDataPackageServiceIP)
        self.TCPDataPackageStatusExpected = mapping[ServerStatusObject.TCPDataPackageService.TCPDataPackageServiceStatus]

        self.SSLCoTStatus = self.SSLCoTStatusCheck(ServerStatusObject.SSLCoTService.SSLCoTServicePort, ServerStatusObject.SSLCoTService.SSLCoTServiceIP)
        self.SSLCoTStatusExpected = mapping[ServerStatusObject.SSLCoTService.SSLCoTServiceStatus]

        self.SSLDataPackageStatus = self.SSLDataPackageStatusCheck(ServerStatusObject.SSLDataPackageService.SSLDataPackageServicePort, ServerStatusObject.SSLDataPackageService.SSLDataPackageServiceIP)
        self.SSLDataPackageStatusExpected = mapping[ServerStatusObject.SSLDataPackageService.SSLDataPackageServiceStatus]

        self.RestAPIStatus = self.RestAPIStatusCheck(ServerStatusObject.RestAPIService.RestAPIServicePort, ServerStatusObject.RestAPIService.RestAPIServiceIP)
        self.RestAPIStatusExpected = mapping[ServerStatusObject.RestAPIService.RestAPIServiceStatus]

        self.FederationServerStatus = self.FederationServerStatusCheck(ServerStatusObject.FederationServerService.FederationServerServicePort, ServerStatusObject.FederationServerService.FederationServerServiceIP)
        self.FederationServerExpected = mapping[ServerStatusObject.FederationServerService.FederationServerServiceStatus]

    def FederationServerStatusCheck(self, FederationPort, IP):
        import ssl
        from FreeTAKServer.controllers.SSLSocketController import SSLSocketController
        import socket
        try:
            if IP == "0.0.0.0":
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('10.255.255.255', 1))
                IP = s.getsockname()[0]
            else:
                pass
            sock = SSLSocketController().createClientSocket(IP)
            sock.settimeout(5)
            sock.connect((IP, FederationPort))
            # returnMessage = sock.recv(100)
            sock.close()
            return "on"
        except Exception as e:
            print('here')
            print(e)
            return "off"

    def TCPDataPackageStatusCheck(self, TCPDataPackagePort, IP):
        import requests
        try:
            import socket
            if IP == "0.0.0.0":
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('10.255.255.255', 1))
                IP = s.getsockname()[0]
            else:
                pass
            conn = requests.get(f'http://{IP}:{TCPDataPackagePort}/Alive')
            if conn.status_code == 200:
                return "on"
            else:
                return "off"
        except Exception as e:
            return "off"

    def TCPCoTStatusCheck(self, TCPCoTPort, IP):
        try:
            import socket
            if IP == "0.0.0.0":
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('10.255.255.255', 1))
                IP = s.getsockname()[0]
            else:
                pass
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((IP, TCPCoTPort))
            sock.send(b'TEST')
            returnMessage = sock.recv(100)
            sock.close()

            if returnMessage == b'success':
                return "on"
            else:
                return "off"
        except Exception as e:
            return "off"

    def SSLDataPackageStatusCheck(self, SSLDataPackagePort, IP):
        import requests
        from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
        try:
            import socket
            if IP == "0.0.0.0":
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('10.255.255.255', 1))
                IP = s.getsockname()[0]
            else:
                pass
            conn = requests.get(f'https://{IP}:{SSLDataPackagePort}/Alive', cert=(MainConfig.pemDir, MainConfig.unencryptedKey), verify=False)
            if conn.status_code == 200:
                return "on"
            else:
                return "off"
        except:
            return "off"

    def SSLCoTStatusCheck(self, SSLCoTPort, IP):
        from FreeTAKServer.controllers.SSLSocketController import SSLSocketController
        try:
            import socket
            if IP == "0.0.0.0":
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('10.255.255.255', 1))
                IP = s.getsockname()[0]
            else:
                pass
            sock = SSLSocketController().createClientSocket(IP)
            sock.settimeout(10)
            sock.connect((IP, SSLCoTPort))
            sock.send(b'TEST')
            returnMessage = sock.recv(100)
            sock.close()
            if returnMessage == b'success':
                return "on"
            else:
                return "off"
        except Exception as e:
            return "off"

    def RestAPIStatusCheck(self, RestAPIPort, IP):
        from eventlet.green.urllib import request
        try:
            import socket
            if IP == "0.0.0.0":
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('10.255.255.255', 1))
                IP = s.getsockname()[0]
            else:
                pass
            conn = request.urlopen(f'http://{IP}:{RestAPIPort}/Alive')
            if conn.code == 200:
                return "on"
            else:
                return "off"
        except:
            return "off"


if __name__ == '__main__':
    status = ServerStatusController(FTS())
    1 == 1


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
#
# __serverdestination.py
# Python implementation of the Class __serverdestination
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):10 AM
# Original author: Corvo
#
#######################################################
from FreeTAKServer.model.FTSModelVariables.ServerdestinationVariables import ServerdestionationVariables as vars

class _Serverdestination(FTSProtocolObject):
    def __init__(self):
        # string composed by IP:port: protocol:machineID. e.g. 192.168.0.103:4242:tcp:
        # ANDROID-R52JB0CDC4E
        self.destinations = None
        # destinations getter

    @staticmethod
    def geochat(DESTINATIONS = vars.geochat().DESTINATIONS):
        serverdestinations = _Serverdestination()
        serverdestinations.setdestinations(DESTINATIONS)
        return serverdestinations

    def getdestinations(self):
        return self.destinations

    # destinations setter
    def setdestinations(self, destinations=None):
        self.destinations = destinations


#######################################################
# 
# FederationServiceAbstract.py
# Python implementation of the Class FederationServiceAbstract
# Generated by Enterprise Architect
# Created on:      12-Sep-2020 10:32:32 PM
# Original author: natha
# 
#######################################################
from FreeTAKServer.model.ServiceObjects.FederateClients import FederateClients
from FreeTAKServer.model.ServiceObjects.Federate import Federate


class FederationServiceAbstract:
    """FederationServiceAbstract.py Python implementation of the Class
    FederationServiceAbstract Generated by Enterprise Architect Created on:      12-
    Sep-2020 10:32:32 PM Original author: natha
    """
    def __init__(self, pipe):
        self.federateClients = FederateClients()
        self.pipe = pipe


    def recv_data_from_clients(self, clients):
        """this function receives data from all connected client
        """
        try:
            if isinstance(clients, list):
                for client in clients:
                    if isinstance(client, Federate):
                        try:
                            for CoT in client.federationController.receive_data_from_federates():
                                self.pipe.send(CoT)
                        except TypeError:
                            self.federateClients.remove_client(client)
                    else:
                        raise TypeError('the list should only contain instances of the Federate class')
                return 1
            else:
                raise TypeError('this function should only be supplied a list object')
        except Exception as e:
            return -1
    def send_data_to_clients(self, data, clients):
        """this function will send the data of a CoT object to all connected client
        """
        try:
            if isinstance(clients, list):
                for client in clients:
                    if isinstance(client, Federate):
                        client.federationController.send_data_to_federates(data)
                    else:
                        raise TypeError('the list should only contain instances of the Federate class')
                return 1
            else:
                raise TypeError('this function should only be supplied a list object')
        except Exception as e:
            return -1

    def check_if_data_is_available(self, clients):
        """
        this function serves a purpose similar to poll
        within a pipe by iterating over clients and
        in the event any data is received the function
        returns this data
        :param clients:
        :return:
        """
        data = []
        try:
            if isinstance(clients, list):
                client = clients[0]
                if isinstance(client, Federate):
                    for CoT in client.federationController.receive_data_from_federates():
                        data.append(CoT)

                else:
                    raise TypeError('the list should only contain instances of the Federate class')
                return data
            else:
                raise TypeError('this function should only be supplied a list object')
        except Exception as e:
            return -1



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# Mission.py
# Python implementation of the Class Mission
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):08 AM
# Original author: Corvo
# 
#######################################################
from FreeTAKServer.model.FTSModelVariables.MissionVariables import MissionVariables as vars
from FreeTAKServer.model.FTSModel.MissionChanges import MissionChanges

class Mission(FTSProtocolObject):
    def __init__(self):
        self.type = None

        self.tool = None

        self.name = None

        self.authorUid = None

    @staticmethod
    def ExcheckUpdate(TYPE=vars.ExcheckUpdate().TYPE, TOOL=vars.ExcheckUpdate().TOOL,
                              NAME=vars.ExcheckUpdate().NAME,
                              AUTHORUID=vars.ExcheckUpdate().AUTHORUID, ):
        mission = Mission()

        mission.settype(TYPE)

        mission.settool(TOOL)

        mission.setname(NAME)

        mission.setauthorUid(AUTHORUID)

        mission.MissionChanges = MissionChanges.ExcheckUpdate()

        return mission

    def settype(self, type):
        self.type = type

    def gettype(self):
        return self.type

    def settool(self, tool):
        self.tool = tool

    def gettool(self):
        return self.tool

    def setname(self, name):
        self.name = name

    def getname(self):
        return self.name

    def setauthorUid(self, authorUid):
        self.authorUid = authorUid

    def getauthorUid(self):
        return self.authorUid

if __name__ == "__main__":
    y = Mission.ExcheckUpdate()


from abc import ABC
class SpecificCoTAbstract(ABC):
    def define_variables(self):
        self.type = None
        self.clientInformation = None
        self.modelObject = None
        self.xmlString = None
    
    def setType(self, type):
        self.type = type
        
    def getType(self):
        return self.type
    
    def setXmlString(self, xmlString):
        self.xmlString = xmlString
        
    def getXmlString(self):
        return self.xmlString
    
    def setClientInformation(self, clientInformation):
        self.clientInformation = clientInformation
        
    def getClientInformation(self):
        return self.clientInformation
    
    def setModelObject(self, modelObject):
        self.modelObject = modelObject
        
    def getModelObject(self):
        return self.modelObject



from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class RestAPIServiceVariables:
    def __init__(self):
        self.RestAPIServiceIP = MainConfig.APIIP
        self.RestAPIServicePort = MainConfig.APIPort
        self.RestAPIServiceStatus = ""


class UidVariables:
    def __init__(self):
        self.DROID = None
        self.INTAG = None

    @classmethod
    def connection(cls):
        cls.DROID = None
        return cls

    @classmethod
    def UserUpdate(cls):
        cls.Droid = None
        return cls

    @classmethod
    def ExcheckUpdate(cls):
        cls.INTAG = None
        return cls
    @classmethod
    def Checklist(cls):
        cls.INTAG = None
        return cls


from FreeTAKServer.controllers.DatabaseControllers.table_controllers import TableController
from FreeTAKServer.model.SQLAlchemy.Event import Event
import importlib

class EventTableController(TableController):
    def __init__(self):
        self.table = Event

    def create(self, session, object):
        row = self.convert_model_to_row(modelObject=object)
        session.add(row)
        session.commit()
        return row

    def convert_model_to_row(self, modelObject, rowObject = None, parentRowObject = None):
        # TODO: find a more elegant way of doing this
        if rowObject == None:
            rowObject = Event()
        else:
            pass
        for attribName, attribValue in modelObject.__dict__.items():
            if hasattr(attribValue, '__dict__'):
                if attribName[0].isalpha():
                    attribName = attribName.capitalize()
                else:
                    attribName = '_'+attribName[1:].capitalize()
                subRowObjectImport = importlib.import_module(
                    f'FreeTAKServer.model.SQLAlchemy.CoTTables.{attribName}')

                subRowObject = getattr(subRowObjectImport, attribName)()
                subTableRowObject = self.convert_model_to_row(attribValue, subRowObject, rowObject)
                #setattr(rowObject, attribName, subTableRowObject)

            elif isinstance(attribValue, list):
                if attribName[0].isalpha():
                    pass
                else:
                    attribName[1].capitalize()
                subRowObjectImport = importlib.import_module(
                    f'FreeTAKServer.model.SQLAlchemy.CoTTables.{attribName.capitalize()}')
                for element in attribValue:
                    subRowObject = getattr(subRowObjectImport, attribName.capitalize())()
                    subTableRowObject = self.convert_model_to_row(element, subRowObject)
                    setattr(subTableRowObject, "owner", rowObject)

            else:
                setattr(rowObject, attribName, attribValue)
        if parentRowObject != None:
            setattr(rowObject, parentRowObject.__class__.__name__, parentRowObject)
        return rowObject


#######################################################
# 
# ClientInformationQueueController.py
# Python implementation of the Class ClientInformationQueueController
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:46:59 AM
# Original author: Natha Paquette
# 
#######################################################
from FreeTAKServer.model.ClientInformationQueue import ClientInformationQueue
from multiprocessing import Queue

class ClientInformationQueueController:
    def __init__(self):  
        self.ClientInformationQueue = ClientInformationQueue()

    def addClientToQueue(self, clientInformation):
        self.ClientInformationQueue.clientQueue.put(clientInformation)

    def removeClientFromQueue(self, clientInformationToRemove):
        #create instance of old queue
        oldQueue = self.ClientInformationQueue.clientQueue
        #create new queue
        newQueue = Queue()
        #iter through len of old queue
        for _ in len(oldQueue):
            #check if retrieved client information matches information to remove if so it will be ignored
            #otherwis it will be added to the new queue
            clientInformation = oldQueue.get()
            if clientInformation == clientInformationToRemove:
                pass
            else:
                newQueue.put(clientInformation)
        self.ClientInformationQueue.clientQueue = newQueue



from setuptools import find_packages, setup
from os import path

this_directory = path.abspath(path.dirname(__file__))

setup(
    name='FreeTAKServer',
    packages=find_packages(include = ['FreeTAKServer', 'FreeTAKServer.*']),
    version='0.1.5.8',
    license='MIT',
    description='An open source server for the TAK family of applications.',

    long_description="Confidential",
    long_description_content_type='text/markdown',
    author='FreeTAKTeam',
    author_email='your.email@domain.com',
    url='https://github.com/FreeTAKTeam/FreeTakServer',
    download_url='https://github.com/Tapawingo/FreeTakServer/archive/v0.8.4-Beta.tar.gz',
    keywords=['TAK', 'OPENSOURCE'],
    install_requires=[
        'flask==1.1.2',
        'lxml==4.6.1',
        'pathlib==1.0.1',
        'tabulate==0.8.7',
        'sqlalchemy==1.3.20',
        'setuptools',
        'Flask_SQLAlchemy==2.4.4',
        'flask-cors==3.0.9',
        'flask-socketio==4.3.1',
        'eventlet==0.29.0',
        'flask_httpauth==4.2.0',
        'protobuf==3.14.0',
        'python-socketio==4.6.0',
        'python-engineio==3.13.2',
        'Jinja2==2.11.2',
        'psutil'
    ],
    extras_require = {'ui': ['FreeTAKServer_UI']},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
)



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.NameVariables import NameVariables as vars
class Name(FTSProtocolObject):
    @staticmethod
    def ExcheckUpdate(INTAG = vars.ExcheckUpdate().INTAG):
        name = Name()
        name.setINTAG(INTAG)
        return name

    @staticmethod
    def Checklist(INTAG = vars.Checklist().INTAG):
        name = Name()
        name.setINTAG(INTAG)
        return name

    def setINTAG(self, INTAG):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG



from FreeTAKServer.model.RestMessages.Chat import Chat

class ChatPost(Chat):
    def __init__(self):
        pass

    def setmessage(self, message):
        self.message = message

    def getmessage(self):
        return self.message

    def setsender(self, sender):
        self.sender = sender

    def getsender(self):
        return self.sender


from .TCPDataPackageServiceVariables import TCPDataPackageServiceVariables as vars

class TCPDataPackageService:
    def __init__(self, TCPDataPackageServiceIP = vars().TCPDataPackageServiceIP,
                 TCPDataPackageServicePort = vars().TCPDataPackageServicePort,
                 TCPDataPackageServiceStatus = vars().TCPDataPackageServiceStatus):
        self.TCPDataPackageServiceIP = TCPDataPackageServiceIP
        self.TCPDataPackageServicePort = TCPDataPackageServicePort
        self.TCPDataPackageServiceStatus = TCPDataPackageServiceStatus


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.CreatorUidVariables import CreatorUidVariables as vars

class CreatorUid(FTSProtocolObject):
    def __init__(self):
        self.INTAG = None

    @staticmethod
    def ExcheckUpdate(INTAG=vars.ExcheckUpdate().INTAG, ):
        creatoruid = CreatorUid()

        creatoruid.setINTAG(INTAG)

        return creatoruid

    def setINTAG(self, INTAG):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG

    @staticmethod
    def Checklist(INTAG=vars.Checklist().INTAG):
        creatorUid = CreatorUid()
        creatorUid.setINTAG(INTAG)
        return creatorUid




#######################################################
# 
# Color.py
# Python implementation of the Class Color
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:35 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String


class Color(Base):
# default constructor  def __init__(self):  
    __tablename__ = "Color"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    argb = Column(String)


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# Link.py
# Python implementation of the Class Link
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):08 AM
# Original author: Corvo
# 
#######################################################
import datetime as dt
from FreeTAKServer.model.FTSModelVariables.LinkVariables import LinkVariables as vars

class Link(FTSProtocolObject):
    def __init__(self):
        self.uid = None
        self.relation = None
        self.production_time = None
        self.type = None
        self.parent_callsign = None

    @staticmethod
    def drop_point(UID = vars.drop_point().UID, RELATION = vars.drop_point().RELATION,
                   PRODUCTIONTIME = vars.drop_point().PRODUCTIONTIME,
                   TYPE = vars.drop_point().TYPE, PARENTCALLSIGN = vars.drop_point().PARENTCALLSIGN):
        link = Link()
        link.setuid(UID)
        link.setrelation(RELATION)
        link.setproduction_time(PRODUCTIONTIME)
        link.settype(TYPE)
        link.setparent_callsign(PARENTCALLSIGN)
        return link

    @staticmethod
    def geochat(UID=vars.geochat().UID, RELATION=vars.geochat().RELATION,
                   PRODUCTIONTIME=vars.geochat().PRODUCTIONTIME, TYPE=vars.geochat().TYPE,
                   PARENTCALLSIGN=vars.geochat().PARENTCALLSIGN):
        link = Link()
        link.setuid(UID)
        link.setrelation(RELATION)
        link.settype(TYPE)
        link.setparent_callsign(PARENTCALLSIGN)
        return link

    @staticmethod
    def emergency_on(UID=vars.emergency_on().UID, RELATION=vars.emergency_on().RELATION,
                PRODUCTIONTIME=vars.emergency_on().PRODUCTIONTIME, TYPE=vars.emergency_on().TYPE,
                PARENTCALLSIGN=vars.emergency_on().PARENTCALLSIGN):
        link = Link()
        link.setuid(UID)
        link.setrelation(RELATION)
        link.setproduction_time(PRODUCTIONTIME)
        link.settype(TYPE)
        link.setparent_callsign(PARENTCALLSIGN)
        return link

    @staticmethod
    def disconnect(UID=vars.disconnect().uid, TYPE=vars.disconnect().type, RELATION=vars.disconnect().relation):
        link = Link()
        link.setuid(UID)
        link.settype(TYPE)
        link.setrelation(RELATION)
        return link

    # uid getter 
    def getuid(self): 
        return self.uid 

    # uid setter 
    def setuid(self, uid=0):  
        self.uid=uid 

    # production_time getter 
    def getproduction_time(self): 
        return self.production_time 

    # production_time setter 
    def setproduction_time(self, production_time=0):
        DATETIME_FMT = "%Y-%m-%dT%H:%M:%SZ"
        if production_time == None:
            timer = dt.datetime
            now = timer.utcnow()
            zulu = now.strftime(DATETIME_FMT)
            add = dt.timedelta(minutes=1)
            production_time_part = dt.datetime.strptime(zulu, DATETIME_FMT) + add
            self.production_time = production_time_part.strftime(DATETIME_FMT)
        else:
            self.production_time = production_time

    # relation getter 
    def getrelation(self): 
        return self.relation 

    # relation setter 
    def setrelation(self, relation=0):  
        self.relation=relation 

    # type getter 
    def gettype(self): 
        return self.type 

    # type setter 
    def settype(self, type=0):  
        self.type=type 

    # parent_callsign getter 
    def getparent_callsign(self): 
        return self.parent_callsign 

    # parent_callsign setter 
    def setparent_callsign(self, parent_callsign=0):  
        self.parent_callsign=parent_callsign 



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
class ChecklistTasks(FTSProtocolObject):
    @staticmethod
    def Checklist():
        checklistTasks = ChecklistTasks()
        return checklistTasks




from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class FederationServerServiceVariables:
    def __init__(self):
        # TODO: change to default ''
        self.FederationServerServiceStatus = 'stop'
        self.FederationServerServicePort = MainConfig.FederationPort
        self.FederationServerServiceIP = "0.0.0.0"


class ContactVariables:
    def __init__(self):
        self.CALLSIGN = None
        self.ENDPOINT = None
        self.ICONSETPATH = None
        self.UID = None
        self.NAME = None

    @classmethod
    def connection(cls):
        cls.CALLSIGN = None
        cls.ENDPOINT = None
        cls.ICONSETPATH = None
        cls.UID = None
        cls.NAME = None
        return cls

    @classmethod
    def drop_point(cls):
        cls.CALLSIGN = "DEFAULT"
        cls.ENDPOINT = None
        cls.ICONSETPATH = None
        cls.UID = None
        cls.NAME = None
        return cls

    @classmethod
    def geochat(cls):
        cls.CALLSIGN = "DEFAULT"
        cls.ENDPOINT = None
        cls.ICONSETPATH = None
        cls.UID = None
        cls.NAME = None
        return cls

    @classmethod
    def emergency_on(cls):
        cls.CALLSIGN = None
        cls.ENDPOINT = None
        cls.ICONSETPATH = None
        cls.UID = None
        cls.NAME = None
        return cls

    @classmethod
    def UserUpdate(cls):
        cls.endpoint = None
        cls.phone = None
        cls.callsign = None
        return cls

    @classmethod
    def SimpleCoT(cls):
        cls.callsign = None
        return cls

    @classmethod
    def Presence(cls):
        cls.callsign = None
        return cls


#######################################################
# 
# VideoStreamTableController.py
# Python implementation of the Class VideoStreamTableController
# Generated by Enterprise Architect
# Created on:      24-Sep-2020 8:19:58 PM
# Original author: natha
# 
#######################################################
from FreeTAKServer.controllers.DatabaseControllers.table_controllers import TableController
from FreeTAKServer.model.SQLAlchemy.VideoStream import VideoStream

class VideoStreamTableController(TableController):

    def __init__(self):
        self.table = VideoStream


from sqlalchemy.inspection import inspect

class SqlAlchemyObjectController:
    def __init__(self):
        pass

    def convert_sqlalchemy_to_modelobject(self, SqlAlchemyObject, modelObject):
        for attribute in SqlAlchemyObject.__dict__.keys():
            if attribute != '_sa_instance_state' and attribute != 'PrimaryKey' and attribute != 'xmlString' and SqlAlchemyObject.__dict__[attribute] != None:
                try:
                    setter = getattr(modelObject, 'set'+attribute)
                    setter(SqlAlchemyObject.__dict__[attribute])
                except Exception as e:
                    print(str(e))
            else:
                pass

        for relationship in SqlAlchemyObject.__mapper__.relationships:
            try:
                y = str(relationship)
                relationship = '.'.join(str(relationship).split('.', 1)[1::])
                relationshipObject = getattr(SqlAlchemyObject, relationship)
                if relationshipObject != None:
                    modelObjectInstance = getattr(modelObject, relationship)
                    setter = getattr(modelObject, 'set'+relationship)
                    setter(self.convert_sqlalchemy_to_modelobject(relationshipObject, modelObjectInstance))
                else:
                    pass
            except Exception as e:
                print(str(e))
        return modelObject
if __name__ == "__main__":
    from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController

    contr = DatabaseController().query_ActiveEmergency()
    x = contr[0]
    from FreeTAKServer.model.FTSModel.Event import Event
    modelobj = Event.emergecyOn()

    y = SqlAlchemyObjectController().convert_sqlalchemy_to_modelobject(x.event, modelobj)
    1==1


class ArgumentConstants:
    def __init__(self):
        self.APIPORTDESC = 'the port address you would like FreeTAKServer to run receive datapackages on not'
        self.IPDESC = "the IP you would like FreeTAKServer to run receive connections on, must be set for private DataPackages to function"
        self.COTPORTDESC = 'the port you would like FreeTAKServer to run receive connections on'
        self.FULLDESC = 'FreeTAKServer startup settings'


#######################################################
# 
# emergency.py
# Python implementation of the Class emergency
# Generated by Enterprise Architect
# Created on:      28-Sep-2020 10:48:15 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy import ForeignKey
from sqlalchemy.orm import relationship


class Emergency(Base):
    __tablename__ = "Emergency"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="emergency")
    Alert = Column(String)
    # if true the emergency beacon is canceled
    cancel = Column(String)
    # default constructor  def __init__(self):
    type = Column(String)


from .FederationClientServiceVariables import FederationClientServiceVariables as vars

class FederationClientService:
    def __init__(self, FederationClientServiceIP = vars().FederationClientServiceIP, 
                 FederationClientServicePort = vars().FederationClientServicePort, 
                 FederationClientServiceStatus = vars().FederationClientServiceStatus):
        self.FederationClientServiceIP = FederationClientServiceIP
        self.FederationClientServicePort = FederationClientServicePort
        self.FederationClientServiceStatus = FederationClientServiceStatus



#######################################################
# 
# uid.py
# Python implementation of the Class uid
# Generated by Enterprise Architect
# Created on:      28-Sep-2020 10:48:30 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy.orm import relationship


class Uid(Base):
# default constructor  def __init__(self):  
    __tablename__ = "Uid"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="uid")
    # TBD, maybe from Android?
    Droid = Column(String)
    version = Column(String)


import datetime as dt

class templateInstanceContentsData:
    def __init__(self):
        self.uid = ""
        self.filename = ""
        self.keywords = ["name", "description", "Callsign"]
        self.mimeType = "application/xml"
        self.name = ""
        self.submissionTime = ""
        self.submitter = ""
        self.hash = ""
        self.size = 0
        self.tool = ""
        self.creatorCallsign = ""

    def setfilename(self, filename = None):
        if filename is None:
            self.filename = self.uid+'.xml'
        else:
            self.filename = filename

    def getfilename(self):
        return self.filename

    def setname(self, name = None):
        if name is None:
            self.name = self.uid
        else:
            self.name = name

    def getname(self):
        return self.name

    def setsize(self, size):
        self.size = size

    def getsize(self):
        return self.size

    def setsubmitter(self, submitter):
        self.submitter = submitter

    def getsubmitter(self):
        return self.submitter

    def sethash(self, hash):
        self.hash = hash

    def gethash(self):
        return self.hash

    def setmimeType(self, mimeType):
        self.mimeType = mimeType

    def getmimeType(self):
        return self.mimeType

    def setkeywords(self, name = None, description = None, callsign = None):
        self.keywords[0] = name
        self.keywords[1] = description
        self.keywords[2] = callsign

    def settool(self,tool):
        self.tool = tool

    def setuid(self, uid):
        self.uid = uid

    def setsubmissionTime(self, timestamp = None):
        DATETIME_FMT = "%Y-%m-%dT%H:%M:%S.%fZ"
        if timestamp == None:
            timer = dt.datetime
            now = timer.utcnow()
            zulu = now.strftime(DATETIME_FMT)
            self.submissionTime = zulu
        else:
            self.submissionTime = timestamp


#######################################################
# 
# Remarks.py
# Python implementation of the Class Remarks
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:41 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy.orm import relationship


class Remarks(Base):
# default constructor  def __init__(self):
    __tablename__ = "Remarks"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="remarks")
    # Used to track a conversation thread.  The format is a comma-separated list of
    # freetext keywords.
    # 
    #               ex. keywords="debriefing"            - Describes a conversation
    # about debriefing
    #               ex. keywords="mission-A"             - Describes a conversation
    # about mission-A
    #               ex. keywords="tasking_B, subject_C"  - Describes a conversation
    # about tasking_B and subject_C
    keywords = Column(String)
    # Source specifies the sender’s UID – this is what is parsed by recipients to
    # determine the sender, with the UID format being the fallback.
    source = Column(String)
    # the time of the remark was added to the CoT object
    time = Column(String)
    # Intended recipeint(s) of this remark information. Tentative field coding as
    # follows: The to attribute may contain the UID of the entity to whom the message
    # is addressed.  (Implementors should expect that future versions of this sub
    # schema will allow a comma separated list of UIDs.)  Absense of an explict
    # addressee means the message is broadcast.
    # e.g. ANDROID-359975090666199
    to = Column(String)
    # Version tag for this sub schema.  Neccessary to ensure upward compatibility
    # with future revisions.
    version = Column(String)


class FederationClientServiceVariables:
    def __init__(self):
        self.FederationClientServiceStatus = ''
        self.FederationClientServicePort = 9000
        self.FederationClientServiceIP = None


#######################################################
# 
# track.py
# Python implementation of the Class track
# Generated by Enterprise Architect
# Created on:      28-Sep-2020 10:48:26 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy import ForeignKey
from sqlalchemy.orm import relationship

class Track(Base):
# default constructor  def __init__(self):  
    __tablename__ = "Track"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="track")
    course = Column(String)
    # 1-sigma error on a Gaussian distribution associated with the course attribute
    eCourse = Column(String)
    # 1-sigma error on a Gaussian distribution associated with the slope attribute
    eSlope = Column(String)
    # 1-sigma error on a Gaussian distribution associated with the speed attribute
    eSpeed = Column(String)
    speed = Column(String)
    version = Column(String)


#######################################################
# 
# MessageTypeController.py
# Python implementation of the Class MessageTypeController
# Generated by Enterprise Architect
# Created on:      20-May-2020 1:09:05 PM
# Original author: Natha Paquette
# 
#######################################################


class MessageTypeController:
    def __init__(self):  
        pass

    def callMessageController(self, data, messageType):
        pass

    def determineMessageType(self, data):
        pass


class SummaryVariables:
    def __init__(self):
        self.INTAG = None

    @classmethod
    def drop_point(cls):
        #appears to be present on points shared over gottena
        cls.INTAG = None
        return cls


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: takmessage.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import FreeTAKServer.model.protobufModel.cotevent_pb2 as cotevent__pb2
import FreeTAKServer.model.protobufModel.takcontrol_pb2 as takcontrol__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='takmessage.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10takmessage.proto\x1a\x0e\x63otevent.proto\x1a\x10takcontrol.proto\"J\n\nTakMessage\x12\x1f\n\ntakControl\x18\x01 \x01(\x0b\x32\x0b.TakControl\x12\x1b\n\x08\x63otEvent\x18\x02 \x01(\x0b\x32\t.CotEventb\x06proto3'
  ,
  dependencies=[cotevent__pb2.DESCRIPTOR,takcontrol__pb2.DESCRIPTOR,])




_TAKMESSAGE = _descriptor.Descriptor(
  name='TakMessage',
  full_name='TakMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='takControl', full_name='TakMessage.takControl', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cotEvent', full_name='TakMessage.cotEvent', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=54,
  serialized_end=128,
)

_TAKMESSAGE.fields_by_name['takControl'].message_type = takcontrol__pb2._TAKCONTROL
_TAKMESSAGE.fields_by_name['cotEvent'].message_type = cotevent__pb2._COTEVENT
DESCRIPTOR.message_types_by_name['TakMessage'] = _TAKMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TakMessage = _reflection.GeneratedProtocolMessageType('TakMessage', (_message.Message,), {
  'DESCRIPTOR' : _TAKMESSAGE,
  '__module__' : 'takmessage_pb2'
  # @@protoc_insertion_point(class_scope:TakMessage)
  })
_sym_db.RegisterMessage(TakMessage)


# @@protoc_insertion_point(module_scope)



from FreeTAKServer.controllers.SpecificCoTControllers.SendCoTAbstractController import SendCoTAbstractController
from FreeTAKServer.model.FTSModel.Event import Event
from FreeTAKServer.model.SpecificCoT.SendEmergency import SendEmergency
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController

class SendEmergencyController(SendCoTAbstractController):
    def __init__(self, RawCoT):
        if RawCoT.status == 'on':
            tempObject = super().Event.emergecyOn()
            object = SendEmergency()
            object.status = 'on'
            self.fill_object(object, tempObject, RawCoT, addToDB=False)
            obj = self.getObject()
            DatabaseController().create_ActiveEmergency(obj.modelObject)

        elif RawCoT.status == 'off':
            tempObject = super().Event.emergecyOff()
            object = SendEmergency()
            object.status = 'off'
            self.fill_object(object, tempObject, RawCoT, addToDB=False)
            obj = self.getObject()
            DatabaseController().remove_ActiveEmergency(query=f'uid == "{obj.modelObject.uid}"')


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
#
# Takv.py
# Python implementation of the Class Takv
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):09 AM
# Original author: Corvo
#
#######################################################
from FreeTAKServer.model.FTSModelVariables.TakvVariables import TakvVariables as vars

class Takv(FTSProtocolObject):

# default constructor
  def __init__(self):
    # the version of TAK running on the device
    self.version = None
    # the variant of TAK
    self.platform = None
    # type of physical device
    self.device = None
    # the operating system running TAK
    self.os = None

  @staticmethod
  def connection(VERSION = vars.connection().VERSION, PLATFORM = vars.connection().PLATFORM,
                 DEVICE = vars.connection().DEVICE, OS = vars.connection().OS):

    takv = Takv()
    takv.setversion(VERSION)
    takv.setplatform(PLATFORM)
    takv.setos(OS)
    takv.setdevice(DEVICE)
    return takv

  @staticmethod
  def UserUpdate(DEVICE=vars.UserUpdate().device, PLATFORM=vars.UserUpdate().platform, OS=vars.UserUpdate().os,
                 VERSION=vars.UserUpdate().version):
    takv = Takv()
    takv.setdevice(DEVICE)
    takv.setplatform(PLATFORM)
    takv.setos(OS)
    takv.setversion(VERSION)
    return takv

    # os getter
  def getos(self):
    return self.os

    # os setter
  def setos(self,os=0):
    self.os=os


    # version getter
  def getversion(self):
    return self.version

    # version setter
  def setversion(self, version=0):
    self.version=version


  def getplatform(self):
      return self.platform

    # platform setter
  def setplatform(self, platform=0):
      self.platform=platform

  # device getter
  def getdevice(self):
    return self.device

  # device setter
  def setdevice(self, device=0):
    self.device=device



#######################################################
# 
# User.py
# Python implementation of the Class User
# Generated by Enterprise Architect
# Created on:      21-Sep-2020 10:28:08 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy import Integer
from sqlalchemy import ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.orm import backref
from FreeTAKServer.model.SQLAlchemy.Event import Event
from FreeTAKServer.model.SQLAlchemy.system_user import SystemUser

class User(Base):
# default constructor  def __init__(self):  

    """sqlalchemy object of table of connected users and their information
    """
    __tablename__ = "User"
    uid = Column(String(25), primary_key = True)
    callsign = Column(String)
    CN = Column(String, nullable=True)
    IP = Column(String)
    CoT_id = Column(String, ForeignKey("Event.uid"))
    CoT = relationship(Event, uselist=False, cascade="all, delete", backref="User")


#######################################################
# 
# orchestrator.py
# Python implementation of the Class orchestrator
# Generated by Enterprise Architect
# Created on:      21-May-2020 12:24:48 PM
# Original author: Natha Paquette
# 
#######################################################
import os
from FreeTAKServer.controllers.ReceiveConnections import ReceiveConnections
from FreeTAKServer.controllers.ClientInformationController import ClientInformationController
from FreeTAKServer.controllers.ClientSendHandler import ClientSendHandler
from FreeTAKServer.controllers.SendClientData import SendClientData
from FreeTAKServer.controllers.DataQueueController import DataQueueController
from FreeTAKServer.controllers.ClientInformationQueueController import ClientInformationQueueController
from FreeTAKServer.controllers.ActiveThreadsController import ActiveThreadsController
from FreeTAKServer.controllers.ReceiveConnectionsProcessController import ReceiveConnectionsProcessController
from FreeTAKServer.controllers.MainSocketController import MainSocketController
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
from FreeTAKServer.controllers.SendDataController import SendDataController
from FreeTAKServer.controllers.AsciiController import AsciiController
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants as DPConst
from FreeTAKServer.model.RawCoT import RawCoT
from FreeTAKServer.controllers.SpecificCoTControllers.SendDisconnectController import SendDisconnectController
from FreeTAKServer.controllers.configuration.OrchestratorConstants import OrchestratorConstants
from FreeTAKServer.controllers.serializers.SqlAlchemyObjectController import SqlAlchemyObjectController
from FreeTAKServer.model.FTSModel.Event import Event

ascii = AsciiController().ascii
from logging.handlers import RotatingFileHandler
import logging
import multiprocessing
import importlib
import sqlite3
import socket

loggingConstants = LoggingConstants()

from FreeTAKServer.controllers.ClientReceptionHandler import ClientReceptionHandler


class Orchestrator:
    #TODO: fix repeat attempts to add user
    # default constructor  def __init__(self):
    def __init__(self):
        log_format = logging.Formatter(loggingConstants.LOGFORMAT)
        self.logger = logging.getLogger(loggingConstants.LOGNAME)
        self.logger.setLevel(logging.DEBUG)
        self.logger.addHandler(self.newHandler(loggingConstants.DEBUGLOG, logging.DEBUG, log_format))
        self.logger.addHandler(self.newHandler(loggingConstants.ERRORLOG, logging.ERROR, log_format))
        self.logger.addHandler(self.newHandler(loggingConstants.INFOLOG, logging.INFO, log_format))
        # create necessary queues
        self.clientInformationQueue = []
        # this contains a list of all pipes which are transmitting CoT from clients
        self.pipeList = []
        # Internal Pipe used for CoT generated by the server itself
        self.internalCoTArray = []
        self.ClientReceptionHandlerEventPipe = ''
        # health check events
        self.healthCheckEventArray = []
        # instantiate controllers
        self.ActiveThreadsController = ActiveThreadsController()
        self.ClientInformationController = ClientInformationController()
        self.ClientInformationQueueController = ClientInformationQueueController()
        self.ClientSendHandler = ClientSendHandler()
        self.DataQueueController = DataQueueController()
        self.ReceiveConnections = ReceiveConnections()
        self.ReceiveConnectionsProcessController = ReceiveConnectionsProcessController()
        self.MainSocketController = MainSocketController()
        self.XMLCoTController = XMLCoTController()
        self.SendClientData = SendClientData()
        self.KillSwitch = 0
        self.openSockets = 0
        self.openSocketsArray = []

    def clear_user_table(self):
        self.dbController.remove_user()
        print('user table cleared')
    def testing(self):
        """
        function which creates variables for testing
        """
        from multiprocessing import Pipe
        from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
        self.dbController = DatabaseController()
        self.CoTSharePipe, other = Pipe()
        return None
    def newHandler(self, filename, log_level, log_format):
        handler = RotatingFileHandler(
            filename,
            maxBytes=loggingConstants.MAXFILESIZE,
            backupCount=loggingConstants.BACKUPCOUNT
        )
        handler.setFormatter(log_format)
        handler.setLevel(log_level)
        return handler

    def sendUserConnectionGeoChat(self, clientInformation):
        # TODO: refactor as it has a proper implementation of a PM to a user generated by the server
        '''
        function to create and send pm to connecting user
        :param clientInformation:
        :return:
        '''
        from FreeTAKServer.controllers.SpecificCoTControllers.SendGeoChatController import SendGeoChatController
        from FreeTAKServer.model.RawCoT import RawCoT
        from FreeTAKServer.model.FTSModel.Dest import Dest
        import uuid
        if OrchestratorConstants().DEFAULTCONNECTIONGEOCHATOBJ != None:
            ChatObj = RawCoT()
            ChatObj.xmlString = f'<event><point/><detail><remarks>{OrchestratorConstants().DEFAULTCONNECTIONGEOCHATOBJ}</remarks><marti><dest/></marti></detail></event>'
            classobj = SendGeoChatController(ChatObj)
            instobj = classobj.getObject()
            instobj.modelObject.detail._chat.chatgrp.setuid1(clientInformation.modelObject.uid)
            dest = Dest()
            dest.setcallsign(clientInformation.modelObject.detail.contact.callsign)
            instobj.modelObject.detail.marti.setdest(dest)
            instobj.modelObject.detail._chat.setchatroom(clientInformation.modelObject.detail.contact.callsign)
            instobj.modelObject.detail._chat.setparent("RootContactGroup")
            instobj.modelObject.detail._chat.setid(clientInformation.modelObject.uid)
            instobj.modelObject.detail._chat.setgroupOwner("True")
            instobj.modelObject.detail.remarks.setto(clientInformation.modelObject.uid)
            instobj.modelObject.setuid('GeoChat.'+'SERVER-UID.'+clientInformation.modelObject.detail.contact.callsign+'.'+str(uuid.uuid1()))
            instobj.modelObject.detail._chat.chatgrp.setid(clientInformation.modelObject.uid)
            classobj.reloadXmlString()
            SendDataController().sendDataInQueue(None, instobj, self.clientInformationQueue)
            return 1
        else:
            return 1
    def clientConnected(self, rawConnectionInformation):
        try:
            # temporarily broken
            # self.check_for_dead_sockets()
            from FreeTAKServer.controllers.DatabaseControllers.EventTableController import EventTableController
            clientPipe = None
            self.logger.info(loggingConstants.CLIENTCONNECTED)
            clientInformation = self.ClientInformationController.intstantiateClientInformationModelFromConnection(
                rawConnectionInformation, clientPipe)
            if self.checkOutput(clientInformation):
                pass
            else:
                raise Exception('error in the creation of client information')
            self.openSockets += 1
            #breaks ssl
            #self.ClientInformationQueueController.addClientToQueue(clientInformation)
            self.clientInformationQueue.append(clientInformation)
            try:
                if hasattr(clientInformation.socket, 'getpeercert'):
                    cn = "placeholder"
                else:
                    cn = None
                CoTRow = EventTableController().convert_model_to_row(clientInformation.modelObject)
                self.dbController.create_user(uid = clientInformation.modelObject.uid, callsign = clientInformation.modelObject.detail.contact.callsign, IP=clientInformation.IP, CoT = CoTRow, CN = cn)
            except Exception as e:
                print(e)
                self.logger.error(
                    'there has been an error in a clients connection while adding information to the database ' +
                    str(e))
            self.logger.info(loggingConstants.CLIENTCONNECTEDFINISHED + str(clientInformation.modelObject.detail.contact.callsign))
            sock = clientInformation.socket
            clientInformation.socket = None
            self.clientDataPipe.send(['add', clientInformation, self.openSockets])
            clientInformation.socket = sock
            self.sendUserConnectionGeoChat(clientInformation)
            return clientInformation
        except Exception as e:
            self.logger.warning(loggingConstants.CLIENTCONNECTEDERROR + str(e))
            return -1

    def check_for_dead_sockets(self):
        # fix function
        try:
            for sock in self.clientInformationQueue:
                if sock.is_alive():
                    pass
                else:
                    self.clientDisconnected(sock)
            return 1
        except Exception as e:
            self.logger.error("there has been an exception in checking for dead sockets " + str(e))
            return -1

    def emergencyReceived(self, processedCoT):
        try:
            if processedCoT.status == loggingConstants.ON:
                self.internalCoTArray.append(processedCoT)
                self.logger.debug(loggingConstants.EMERGENCYCREATED)
            elif processedCoT.status == loggingConstants.OFF:
                for CoT in self.internalCoTArray:
                    if CoT.type == "Emergency" and CoT.modelObject.uid == processedCoT.modelObject.uid:
                        self.internalCoTArray.remove(CoT)
                        self.logger.debug(loggingConstants.EMERGENCYREMOVED)
        except Exception as e:
            self.logger.error(loggingConstants.EMERGENCYRECEIVEDERROR + str(e))

    def dataReceived(self, RawCoT):
        # this will be executed in the event that the use case for the CoT isnt specified in the orchestrator
        try:
            # this will check if the CoT is applicable to any specific controllers
            RawCoT = self.XMLCoTController.determineCoTType(RawCoT)
            # the following calls whatever controller was specified by the above function
            module = importlib.import_module('FreeTAKServer.controllers.SpecificCoTControllers.' + RawCoT.CoTType)
            CoTSerializer = getattr(module, RawCoT.CoTType)
            #TODO: improve way in which the dbController is passed to CoTSerializer
            RawCoT.dbController = self.dbController
            processedCoT = CoTSerializer(RawCoT).getObject()
            sender = processedCoT.clientInformation
            # this will send the processed object to a function which will send it to connected clients
            '''try:
                # TODO: method of determining if CoT should be added to the internal array should
                #  be improved
                if processedCoT.type == "Emergency":
                    self.emergencyReceived(processedCoT)
                else:
                    pass
            except Exception as e:
                return -1'''
            return processedCoT
        except Exception as e:
            self.logger.error(loggingConstants.DATARECEIVEDERROR + str(e))
            return -1

    def sendInternalCoT(self, client):
        try:
            if len(self.internalCoTArray) > 0:
                for processedCoT in self.internalCoTArray:
                    SendDataController().sendDataInQueue(processedCoT.clientInformation, processedCoT, [client])
            else:
                pass
            self.send_active_emergencys(client)
            return 1
        except Exception as e:
            self.logger.error(loggingConstants.MONITORRAWCOTERRORINTERNALSCANERROR + str(e))
            return -1

    def send_active_emergencys(self, client):
        """
        this function needs to be cleaned up however it's functionality is as follows
        it query's the DB for active emergency's at which point it iterates over all
        emergency objects, transforms them into model objects and then xmlStrings
        finally the object is sent to the client.
        """
        try:

            from FreeTAKServer.model.SpecificCoT.SendEmergency import SendEmergency
            emergencys = self.dbController.query_ActiveEmergency()
            for emergency in emergencys:
                emergencyobj = SendEmergency()
                modelObject = Event.emergecyOn()

                filledModelObject = SqlAlchemyObjectController().convert_sqlalchemy_to_modelobject(emergency.event, modelObject)
                emergencyobj.setXmlString(XMLCoTController().serialize_model_to_CoT(filledModelObject))
                emergencyobj.setModelObject(filledModelObject)
                SendDataController().sendDataInQueue(None, emergencyobj, [client])

        except Exception as e:
            self.logger.error('an exception has been thrown in sending active emergencies ' + str(e))

    def clientDisconnected(self, clientInformation):
        if hasattr(clientInformation, 'clientInformation'):
            clientInformation = clientInformation.clientInformation
        else:
            pass
        try:
            try:
                self.ActiveThreadsController.removeClientThread(clientInformation)
                self.dbController.remove_user(query=f'uid == "{clientInformation.modelObject.uid}"')
            except Exception as e:
                self.logger.error('there has been an error in a clients disconnection while adding information to the database')
                pass
            self.openSockets -= 1
            socketa = clientInformation.socket
            clientInformation.socket = None
            self.clientDataPipe.send(['remove', clientInformation, self.openSockets])
            clientInformation.socket = socketa
            try:
                clientInformation.socket.shutdown(socket.SHUT_RDWR)
            except Exception as e:
                self.logger.error('error shutting socket down in client disconnection')
                pass
            try:
                clientInformation.socket.close()
            except Exception as e:
                self.logger.error('error closing socket in client disconnection')
                pass

            self.logger.info(loggingConstants.CLIENTDISCONNECTSTART)
            for client in self.clientInformationQueue:
                if client.ID == clientInformation.ID:
                    self.clientInformationQueue.remove(client)
                else:
                    pass
            try:
                self.ActiveThreadsController.removeClientThread(clientInformation)
                self.dbController.remove_user(query=f'uid == "{clientInformation.modelObject.uid}"')
            except Exception as e:
                self.logger.error('there has been an error in a clients disconnection while adding information to the database')
                pass
            # TODO: remove string
            tempXml = RawCoT()
            tempXml.xmlString = '<event><detail><link uid="{0}"/></detail></event>'.format(clientInformation.modelObject.uid).encode()
            disconnect = SendDisconnectController(tempXml)
            SendDataController().sendDataInQueue(disconnect.getObject().clientInformation, disconnect.getObject(), self.clientInformationQueue, self.CoTSharePipe)
            self.logger.info(loggingConstants.CLIENTDISCONNECTEND + str(clientInformation.modelObject.detail.contact.callsign))
            return 1
        except Exception as e:
            self.logger.error(loggingConstants.CLIENTCONNECTEDERROR + " " + str(e))
            pass

    def monitorRawCoT(self,data):
        # this needs to be the most robust function as it is the keystone of the program
        # this will attempt to define the type of CoT along with the designated controller
        try:
            if isinstance(data, int):
                return None
            else:
                CoT = XMLCoTController().determineCoTGeneral(data)
                function = getattr(self, CoT[0])
                output = function(CoT[1])
                return output
        except Exception as e:
            self.logger.error(loggingConstants.MONITORRAWCOTERRORB + str(e))
            return -1

    def checkOutput(self, output):
        if output != -1 and output != None:
            return True
        else:
            return False

    def loadAscii(self):
        ascii()

    def mainRunFunction(self, clientData, receiveConnection, sock, pool, event, clientDataPipe, ReceiveConnectionKillSwitch, CoTSharePipe, ssl = False):
        print('server started')
        import datetime
        receiveconntimeoutcount = datetime.datetime.now()
        lastprint = datetime.datetime.now()
        while event.is_set():
            self.CoTSharePipe = CoTSharePipe
            try:
                if ssl == True:
                    pass
                self.clientDataPipe = clientDataPipe
                if event.is_set():
                    try:
                        if ReceiveConnectionKillSwitch.is_set():
                            try:
                                receiveConnection.successful()
                            except:
                                pass
                            ReceiveConnectionKillSwitch.clear()
                            receiveConnection = pool.apply_async(ReceiveConnections().listen,
                                                                 (sock,))
                        else:
                            receiveConnectionOutput = receiveConnection.get(timeout=0.01)
                            receiveConnection = pool.apply_async(ReceiveConnections().listen, (sock, ssl,))
                            receiveconntimeoutcount = datetime.datetime.now()
                            lastprint = datetime.datetime.now()
                            CoTOutput = self.handel_connection_data(receiveConnectionOutput)

                    except multiprocessing.TimeoutError:

                        if (datetime.datetime.now() - receiveconntimeoutcount) > datetime.timedelta(seconds=60) and ssl == True:
                            from multiprocessing.pool import ThreadPool
                            try:
                                pass
                                print('\n\nresetting\n\n')
                                pool.terminate()
                                pool = ThreadPool(processes=2)
                                self.pool = pool
                                receiveconntimeoutcount = datetime.datetime.now()
                                lastprint = datetime.datetime.now()
                                clientData = pool.apply_async(ClientReceptionHandler().startup,
                                                              (self.clientInformationQueue,))
                                receiveConnection = pool.apply_async(ReceiveConnections().listen, (sock, ssl,))
                            except Exception as e:
                                print(str(e))
                        elif ssl == True and (datetime.datetime.now() - lastprint) > datetime.timedelta(seconds=30):
                            print('time since last reset ' + str(datetime.datetime.now() - receiveconntimeoutcount))
                            lastprint = datetime.datetime.now()
                        else:
                            pass
                    except Exception as e:
                        self.logger.error('exception in receive connection within main run function '+str(e))

                    try:
                        clientDataOutput = clientData.get(timeout=0.01)
                        if self.checkOutput(clientDataOutput) and isinstance(clientDataOutput, list):
                            CoTOutput = self.handel_regular_data(clientDataOutput)
                        else:
                            raise Exception('client reception handler has returned data which is not of type list data is ' + str(clientDataOutput))
                        clientData = pool.apply_async(ClientReceptionHandler().startup, (self.clientInformationQueue,))
                    except multiprocessing.TimeoutError:
                        pass
                    except Exception as e:
                        self.logger.info('exception in receive client data within main run function ' + str(e))
                        pass
                    try:
                        if CoTSharePipe.poll(timeout=0.1):
                            data = CoTSharePipe.recv()
                            CoTOutput = self.handel_shared_data(data)
                        else:
                            pass
                    except Exception as e:
                        self.logger.error('there has been an excepion in the handling of data supplied by the rest API '+str(e))
                        pass
                else:
                    self.stop()
                    break
            except Exception as e:
                self.logger.info('there has been an uncaught error thrown in mainRunFunction' + str(e))
                pass
        self.stop()

    def handel_shared_data(self, modelData):
        try:
            print('data received within orchestrator '+str(modelData.xmlString))
            if hasattr(modelData, 'clientInformation'):
                output = SendDataController().sendDataInQueue(modelData.clientInformation, modelData,
                                                              self.clientInformationQueue)
            #

            elif modelData.type == "connmessage":
                self.internalCoTArray.append(modelData)

            # this runs in the event of a new connection
            else:
                output = SendDataController().sendDataInQueue(None, modelData,
                                                              self.clientInformationQueue)
        except Exception as e:
            print(e)

    def handel_regular_data(self, clientDataOutput):
        try:
            for clientDataOutputSingle in clientDataOutput:
                try:
                    print('handling reg data')
                    if clientDataOutputSingle == -1:
                        continue
                    CoTOutput = self.monitorRawCoT(clientDataOutputSingle)
                    if CoTOutput == 1:
                        continue
                    elif self.checkOutput(CoTOutput):
                        output = SendDataController().sendDataInQueue(CoTOutput.clientInformation, CoTOutput,
                                                                      self.clientInformationQueue, self.CoTSharePipe)
                        if self.checkOutput(output) and isinstance(output, tuple) == False:
                            pass
                        elif isinstance(output, tuple):
                            self.logger.error('issue sending data to client now disconnecting')
                            self.clientDisconnected(output[1])

                        else:
                            self.logger.error('send data failed in main run function with data ' + str(
                                CoTOutput.xmlString) + ' from client ' + CoTOutput.clientInformation.modelObject.detail.contact.callsign)

                    else:
                        raise Exception('error in general data processing')
                except Exception as e:
                    self.logger.info(
                        'exception in client data, data processing within main run function ' + str(
                            e) + ' data is ' + str(CoTOutput))
                    pass
                except Exception as e:
                    self.logger.info(
                        'exception in client data, data processing within main run function ' + str(
                            e) + ' data is ' + str(clientDataOutput))
        except Exception as e:
            self.logger.info("there has been an error iterating client data output " + str(e))
            return -1
        return 1

    def handel_connection_data(self, receiveConnectionOutput):
        try:
            print('handling conn data')
            if receiveConnectionOutput == -1:
                return None

            CoTOutput = self.monitorRawCoT(receiveConnectionOutput)
            if CoTOutput != -1 and CoTOutput != None:
                output = SendDataController().sendDataInQueue(CoTOutput, CoTOutput,
                                                              self.clientInformationQueue, self.CoTSharePipe)
                if self.checkOutput(output):
                    self.logger.debug('connection data from client ' + str(
                        CoTOutput.modelObject.detail.contact.callsign) + ' successfully processed')
                else:
                    raise Exception('error in sending data')
            else:
                pass
        except Exception as e:
            self.logger.error('exception in receive connection data processing within main run function ' + str(
                e) + ' data is ' + str(CoTOutput))
            return -1
        self.sendInternalCoT(CoTOutput)
        return 1

    def start(self, IP, CoTPort, Event, clientDataPipe, ReceiveConnectionKillSwitch, RestAPIPipe):
        try:
            self.db = sqlite3.connect(DPConst().DATABASE)
            os.chdir('../../../')
            # create socket controller
            self.MainSocketController.changeIP(IP)
            self.MainSocketController.changePort(CoTPort)
            sock = self.MainSocketController.createSocket()
            #changed
            from multiprocessing.pool import ThreadPool
            pool = ThreadPool(processes=2)
            self.pool = pool
            clientData = pool.apply_async(ClientReceptionHandler().startup, (self.clientInformationQueue,))
            receiveConnection = pool.apply_async(ReceiveConnections().listen, (sock,))
            # instantiate domain model and save process as object
            self.mainRunFunction(clientData, receiveConnection, sock, pool, Event, clientDataPipe, ReceiveConnectionKillSwitch, RestAPIPipe)

        except Exception as e:
            self.logger.critical('there has been a critical error in the startup of FTS' + str(e))
            return -1

    def stop(self):
        self.clientDataPipe.close()
        self.pool.terminate()
        self.pool.close()
        self.pool.join()



"""if __name__ == "__main__":

    parser = argparse.ArgumentParser(description=OrchestratorConstants().FULLDESC)
    parser.add_argument(OrchestratorConstants().COTPORTARG, type=int, help=OrchestratorConstants().COTPORTDESC,
                        default=OrchestratorConstants().COTPORT)
    parser.add_argument(OrchestratorConstants().IPARG, type=str, help=OrchestratorConstants().IPDESC,
                        default=OrchestratorConstants().IP)
    parser.add_argument(OrchestratorConstants().APIPORTARG, type=int, help=OrchestratorConstants().APIPORTDESC,
                        default=DataPackageServerConstants().APIPORT)
    args = parser.parse_args()
    CreateStartupFilesController()
    Orchestrator().start(args.IP, args.CoTPort, args.APIPort)"""